/* file system manager addon for sre-http 
   Note that it is sensitive to the CHECKIT variable, and
   the REMOTE_CONFIG "initfilt.80" variable
*/

allowed_dirs.=''        /* do NOT change this */

/* ---------------- User Changeable Parameters -------------- */

/* list of directories that may be examined -- used if checkit=2.
   ALLOWED_DIRS.0= # of entries. 
   
   ALLOWED_DIRS.n='x:\dir1\  privlist'
           -- if this abbrev matches the actual directory,
              and the client has a privilege in privlist,
              then this directory will be displayed 
              If privlist not specified, or contains an *, then 
              privlist is not checked
  ALLOWED_DIRS.!host.0 = # of host specific entries
  ALLOWED_DIRS.!nost.n = like ALLOWED_DIRS.n, but for this host only

Notes:
 *  for each request, the "default" (ALLOWED_DIRS.n) and
    the host specific (ALLOWED_DIRS.!HOST.n) will be checked,
    where HOST is the host_nickname (this is ignored if the
    request is to the "default" host). 
    Please remember that you MUST place an ! before the host_nickname.  
  * If ALLOWED_DIRS.0=0, then there are NO limitations
  * ALLOWED_DIRS is used ONLY when CHECKIT=2

*/
allowed_dirs.0=''

/* Checkit values are:
   0 = allow remote use by superusers (unless suppressed by SREF_NO_REMOTE_CONFIG)
       to goserve datadir, working directory, and sre virtual directories
       ALLOWED_DIRS. is not checked.
       Note: if SREF_NO_REMOTE_CONFIG=1, then checkit=0 is the same as CHECKIT=1
   1 = do NOT allow remote use. Otherwise, same as checkit=0
   2 = allow remote (and local) access to the ALLOWED_DIRS. only
       If checkit=2, then SREF_NO_REMOTE_CONFIG is ignored           */
CHECKIT=0  


/* Require an encoded secret privliege (a dynamic password) 
   To use this, clients must have a ?MANAGER:a_password "secret privilege" 
  1= yes
  0= no */
dynamic_pwd=1

/* duration of dynamic_password, in days (fractions of days okay */
dynpwd_duration=1

/* colors used to display del,copy,move radio buttons
   be sure to include the leading # */
delcolor="#f5ddf9"
copycolor="#00fdf9"
movecolor="#9addf9"
downloadcolor="#778877"

/* maximum size (in byte) of files that can be edited (this is a function of how
large a <TEXTAREA> the target browsers can handle) */
max_size_edit=25000

/* columns and rows to use (in TEXTAREA box) when editing a file (this is a 
   function of how large a <TEXTAREA> the target browsers can handle.) */
col_size_edit=70
row_size_edit=20

/* Name of "response file" to use for dynamic passwords. If blank, use
   generic */
respfile=''       

/* ---------------- End of user changeable parameters  ------------------ */

/* ---------------- DO NOT MODIFY BELOW THIS LINE  ------------------ */


parse arg ddir,tempfile,sel,list,verb,uri,user,basedir,workdir,privset0, ,
           enmadd,transaction,verbose,servername,host_nickname,homedir

if verb="" then do
   say " This SRE-http procedure is not meant to be run in stand-alone mode. "
   exit
end  /* Do */

signal on error name badguys; signal on syntax name badguys
crlf = '0d0a'x


parse var uri manname '?' .
manname='/'||strip(manname,'l','/')


foo=rxfuncquery('UZLoadFuncs')
if foo=1 then do
     call RxFuncAdd 'UZLoadFuncs', 'UNZIPAPI', 'UZLoadFuncs'
     call UZLoadFuncs
end
foo=rxfuncquery('UZLoadFuncs')
candozip=1
if foo=1 then candozip=0


parse var privset0 privset1 ',' privset2
privset=privset1||' '||privset2

/* is this guy allowed ? */
res=is_allowed(checkit)
if res<>'' then return res

/* dynamic password required? */
if dynamic_pwd=1  then do
/*
  if macroquery("SREF_DYNAMIC_PWD")='' then do
       'string SRE-http MANAGER: SREF_DYNAMIC_PWD is NOT available '
        exit
  end /
*/
  signal on error name nodyn; signal on syntax name nodyn

  res=sref_dynamic_pwd('MANAGER','/'uri,privset2,dynpwd_duration,respfile,user)
  if res<>'' then return  res
  signal on error name badguys; signal on syntax name badguys

end
  
allowed_dirs.!privset=translate(privset)
allowed_dirs.!nick=host_nickname

if allowed_dirs.0<>' ' then do
    do ii=1 to allowed_dirs.0
         allowed_dirs.ii=strip(translate(allowed_dirs.ii))
    end /* do */
end /* do */
if host_nickname<>'' then do
  ahost='!'||host_nickname
  if allowed_dirs.ahost.0<>' ' then do
       do ii=1 to allowed_dirs.ahost.0
          allowed_dirs.ahost.ii=strip(translate(allowed_dirs.ahost.ii))
       end /* do */
  end /* do */
end

/* possible base directories */

allowed_dirs.!checkit=checkit
if checkit<>2  then do          /* allow access to working,data,and virtual dirs */
   call make_rootlist_all
   allowed_Dirs.0=0
end
else do
  call make_rootlist_allowed   /* allowed access to "allowed" directories only */
end

/* is this an upload? */
call is_upload
if result<>0 then signal alldone

/* not an upload ... */

/* fill in varlist. */
alist=read_vars(list)

a1=strip(varlist.!root)
a1a='$'a1
a2=rootlist.a1a

ddir=a2
using_root=ddir
using_root_alias=a1

subdir=translate(varlist.!dir,'\','/')
if varlist.!dir<>'' & varlist.!dir<>'/' then ddir=ddir'\'||strip(subdir,,'\')

ddir1=ddir'\*.*'
foo=sysfiletree(ddir1,'ddirfiles','FT')
foo=sysfiletree(ddir1,'ddirdirs','DO')

/* write top of response file */

/* no arguments-- make the frameset */
varlist.!mode=translate(varlist.!mode)
select
  when list='' | varlist.!mode='FRAMESET' then do
      call make_frames
  end
   when varlist.!mode="DOWNLOADS" then do
      call do_downloads
  end /* do */
  when varlist.!mode='SAVEFILE' then do
    call save_file
  end
  when varlist.!ZIPFILE<>'' then do
     if candozip=0 then do
        'string uzunzip not available '
        return '200 22'
     end /* do */
     call show_zipfile
  end /* do */
  when varlist.!INZIP<>'' then do 
    if candozip=0 then do
        'string uzunzip not available '
        return '200 22'
     end /* do */
      call send_zipfile
  end /* do */
  when varlist.!UNZIP<>'' then do
     call unzip_zipfile
  end
  when varlist.!file<>'' | varlist.!textfile<>'' then do
      call show_file  
      return result     /* show file will return a result */
  end  
  when translate(varlist.!mode)='TOC' then do
     call make_a_control
  end
  when varlist.!mode='DIRDO' then do
     call dirdo
  end
  when varlist.!mode='LIST' then do
      call make_list 0
  end /* do */
  when varlist.!mode='EXT_LIST' then do
      call make_list 1
  end /* do */
  when varlist.!mode='DATE_LIST' then do
      call make_list 2
  end /* do */
  when varlist.!mode='NAME_LIST' then do
      call make_list 3
  end /* do */

  when varlist.!mode="LOGON" then do
      call relogon
  end /* do */
  when varlist.!mode='SHORTLIST' then do
      call make_short_list 0
  end /* do */
  when varlist.!mode='EXT_SHORTLIST' then do
      call make_short_list 1
  end /* do */
  when varlist.!mode='EXT_NAME_LIST' then do
      call make_short_list 3
  end /* do */
  when varlist.!mode='EXT_DATE_LIST' then do
      call make_short_list 2
  end /* do */



  when varlist.!mode='TREE' then do
       call make_tree
  end /* do */
  when varlist.!mode='TREECOMPACT' then do
       call make_tree_compact
  end /* do */

  when varlist.!mode='DOIT' & varlist.!EDIT=1 then do
    call editit
  end /* do */
  when varlist.!mode='DOIT' then do
     call make_doit1
     if result=1 then  call do_actions
  end /* do */
  when varlist.!mode='DOIT2' then do
      call cvt_actions
      call lineout tempfile,'<html><head><title>SRE-FileManager </title></head>'
      call do_actions
  end /* do */
  otherwise do
      call make_list
  end /* do */
end

/* all done creating output -- ship it */
alldone:
if completed()<>1 then do
   iff=stream(tempfile,'c','query size')
   foo=sref_gos('FILE ERASE type text/html name 'tempfile)
   return '200 'iff
end
return result           /* completed by proc */



/*********************/
/* make rootlist from working, data, and virtual dirs */
make_rootlist_all:

/* possible base directories */
ddir=strip(translate(ddir,'\','/'),'t','\')
servdir=value(enmadd||'SERVDIR',,'os2environment')
rootlist.=''

/* This is the default (in case ROOT is not specified */
aa='$'
rootlist.aa=ddir

/* datadir is always included */
aroot=translate(translate(strip(ddir),'__','\/'))  
aa='$'||aroot
rootlist.aa=ddir
rootlist.0=rootlist.0' 'aroot

/* server directory is always included */
aroot=translate(translate(strip(servdir),'__','\/'))
aa='$'||aroot
rootlist.aa=servdir
rootlist.0=rootlist.0' 'aroot

/* add virtual directories */
vfile=value(enmadd||'VIRTUAL_FILE',,'os2environment')
filenames=SREF_DO_VIRTUAL(,'*LIST*')
parse var filenames nvirts (crlf) filenames ; nvirts=strip(nvirts)
if datatype(nvirts)<>'NUM' then nvirts=0
do mm=1 to nvirts
   parse var filenames aroot adir (crlf) filenames
   aroot=translate(translate(strip(aroot),'__','\/'))
   adir=space(translate(adir,' ','*'||'0d0a0900'),0)
   adir=strip(translate(adir,'\','/'),'t','\')
   aa='$'||aroot
   rootlist.aa=adir
   rootlist.0=rootlist.0' 'aroot
end /* do */

return 0


/*********************/
/* make rootlist from allowed_dirs */
make_rootlist_allowed:

rootlist.=''

/* add allowed dirs  -- default host */
if datatype(allowed_dirs.0)='NUM' then do
  do mm=1 to allowed_dirs.0
    parse var allowed_dirs.mm adir .
    aroot=translate(translate(strip(adir),'__','\/'))
    adir=space(translate(adir,' ','*'||'0d0a0900'),0)
    adir=strip(translate(adir,'\','/'),'t','\')
    aa='$'||aroot
    rootlist.aa=adir
    rootlist.0=rootlist.0' 'aroot
 end /* do */
end

/* add allowed dirs  -- specific host */

defset=0
if rootlist.0<>'' then do       /* set the default root */
   tt='$'||strip(word(rootlist.0,1))
   ttd=rootlist.tt
   aa='$'
   rootlist.aa=ttd
   defset=1
end /* do */

if host_nickname=0 | host_nickname='' then return 1  
if datatype(allowed_dirs.ahost.0)<>'NUM' then return 1
ah='!'||strip(translate(host_nickname))

do mm=1 to allowed_dirs.ahost.0
   parse var allowed_dirs.ahost.mm adir .
   aroot=translate(translate(strip(adir),'__','\/'))
   adir=space(translate(adir,' ','*'||'0d0a0900'),0)
   adir=strip(translate(adir,'\','/'),'t','\')
   aa='$'||aroot
   rootlist.aa=adir
   rootlist.0=rootlist.0' 'aroot
end /* do */


if defset=0 & rootlist.0<>'' then do       /* set the default root */
   tt='$'||strip(word(rootlist.0,1))
   ttd=rootlist.tt
   aa='$'
   rootlist.aa=ttd
end /* do */

return 2



/***********************/
/* save an "edited" file */
save_file:
/* write file list */
call lineout tempfile,'<html><head><title>SRE-FileManager: save edited file </title>'
call lineout tempfile,'</head>'

call lineout tempfile,'<body><h2>Saving an edited file</h2>'
/* do first file with any mark */
okay=1
do mmm=1 to  1
   if abbrev(varlist.!file,'$')=0 then iterate /* ONLY allow "relative" directories */
   parse var varlist.!file '$' atype '\' fname
   atype=translate(atype)
   aa='$'atype
   addme=rootlist.aa
   okay=2
   if addme='' then leave
   fuse=addme'\'fname  
   okay=3
   if is_allowed_dir(fuse)=0 then leave

/* save CONTENTS */
   dastuff=varlist.!contents
   okay=4
   if dastuff='' then leave

   okay=5
   isold=stream(fuse,'c','query exists')
   if varlist.!backup<>1 & isold<>'' then do
       foo=sysfiledelete(isold)
       if foo<>0 then leave
       call lineout tempfile,'Old file deleted: 'isold'<p>'

   end /* do */
   if varlist.!backup=1 & isold<>'' then do
        arf=lastpos('.',isold); if arf=0 then arf=length(isold)
        bkname=left(isold,arf)||'???'
        bkname=dostempname(bkname)
        foo=dosrename(isold,bkname)
        okay=6
        if foo=0 then leave
        call lineout tempfile,'Old file renamed to: 'bkname'<p>'
   end /* do */
   
   foo=stream(fuse,'c','open write')
   okay=7
   if abbrev(translate(foo),'READY')<>1 then leave
   okay=8
   foo=charout(fuse,dastuff,1) 
   foo2=stream(fuse,'c','close')
   if foo>0 then leave
   okay=0
   leave
end


errms.1='No file specifiled'
errms.2='No such directory '
errms.3='Not allowed to access this directory'
errms.4='No contents'
errms.5='Problem deleting old file' 
errms.6='Problem renaming old file' 
errms.7='Problem opening file for write '
errms.8='Problem writing contents to file '
if okay>0 then do
   call lineout tempfile,'<b>Error:</b>'errms.okay
end
else do
   call lineout tempfile,length(dastuff)' bytes saved to 'fuse
end /* do */
call lineout tempfile,'<p><a href="javascript:self.back()">back</a><br>'
call lineout tempfile,'<a href="'manname'?" target="_top">Reload manager </a>'


call lineout tempfile,'</body></html>'
call lineout tempfile
return 0


/***********************/
/* edit a file */
editit:

a_max_size_edit=max_size_edit
a_col_size_edit=col_size_edit
a_row_size_edit=row_size_edit

/* write file list */
call lineout tempfile,'<html><head><title>SRE-FileManager: edit file </title>'
call lineout tempfile,'<script language="javascript">'
vvar=varlist.!dir
if vvar='' then vvar='/'
call lineout tempfile,'var viewing="'vvar'"'
call lineout tempfile,'var rootis="'using_root_alias'"'
call lineout tempfile,'</script>'
call lineout tempfile,'</head>'

call lineout tempfile,'<body><h2>Edit this file</h2>'
/* do first file with any mark */
okay=1
do mmm=1 to varlist.!file.0
   if varlist.!DO.mmm='' then iterate
   if abbrev(varlist.!file.mmm,'$')=0 then iterate /* ONLY allow "relative" directories */
   parse var varlist.!file.mmm '$' atype '\' fname
   atype=translate(atype)
   aa='$'atype
   addme=rootlist.aa
   okay=2
   if addme='' then leave
   fuse=addme'\'fname  
    okay=3
   if is_allowed_dir(fuse)=0 then leave

   infuse=stream(fuse,'c','query size')
   okay=4
   if infuse=0 | infuse='' then leave
   okay=5
   aa=stream(fuse,'c','open read')
   if abbrev(translate(aa),'READY')<>1 then leave
   stuff=charin(FUSE,1,infuse)
   okay=6
   if length(stuff)<>infuse then leave
   foo=stream(fuse,'c','close')

   thefile=varlist.!file.mmm
   extrax=strip(varlist.!DEST.mmM)

   if extrax<>'' then do
     parse var extrax mx1 mx2 mx3 .
     lb=extract('LIMITBODY')
     if mx1='*' then mx1=(lb-1)
     if datatype(mx1)="NUM" then do
        a_max_size_edit=mx1*1000
     end /* do */
     if datatype(mx2)='NUM' then do
        a_col_size_edit=mx2
     end /* do */
     if datatype(mx3)='NUM' then do
        a_row_size_edit=mx3
     end /* do */
   end /* do */
     say a_max_size_edit
   okay=7
   if infuse>a_max_size_edit then leave
   okay=0     /* okay=0 means "no error" */
   leave
end

if okay>0 then do       /* some kind of error */
   errms.1='No file specified'
   errms.3='Not allowed to access this directory'
   errms.2='No such directory'
   errms.4='No such file'
   errms.5='File inaccessible' 
   errms.6='Problem reading file '||LENGTH(STUFF)',' INFUSE
   errms.7='File too large:'||infuse' (max is 'a_max_Size_edit' bytes)'
   call lineout tempfile,'<b>Error:</b>'errms.okay
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 0
end


call lineout tempfile,'<form action="'manname'" method="POST">'
call lineout tempfile,'<input type="hidden" name="MODE" value="SAVEFILE">'
call lineout tempfile,'<input type="hidden" name="FILE" value="'thefile'">'
call lineout tempfile,'Edit 'fuse', and then resubmit it.<p>'
call lineout tempfile,'<textarea ROWS='a_row_size_edit' COLS='a_col_size_edit' name="CONTENTS">'

adesc=sref_replacestrg(STUFF,'&','&amp;','ALL')
adesc=sref_replacestrg(adesc,'<','&lt;','ALL')
adesc=sref_replacestrg(adesc,'>','&gt;','ALL')
adesc=sref_replacestrg(adesc,'"','&quot;','ALL')

call lineout tempfile,ADESC
call lineout tempfile,'</textarea>'
call lineout tempfile,'<input type="submit" value="Save it">'
call lineout tempfile,'<input type="checkbox" name="BACKUP" checked value=1>Rename old version'
call lineout tempfile,'<p><a href="javascript:self.back()">back</a><br>'
call lineout tempfile,'<a href="'manname'?" target="_top">Reload manager </a>'

call lineout tempfile,'</form></body></html>'
call lineout tempfile

return 0


/***********************/
/* logon again */
relogon:

foo=sref_extract_upwd()
parse var foo aname ':' .
aname=strip(aname)
if aname=varlist.!user then do
  'header add WWW-Authenticate: Basic Realm=<Manager>'  /* challenge */
   call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 3.0//EN">'
   call lineout tempfile, "<html><head><title>Logon to MANAGER  </title>"
   call lineout tempfile, '</head><body>'
   call lineout tempfile,' <strong>Please enter a username and pass word (to use MANAGER)</strong> <pre>'
   call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
   call lineout tempfile,' </body> </html> '
   call lineout tempfile
   return sref_response('unauth'," MANAGER: requires a valid username/password ",servername,1)
end
call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 3.0//EN">'
call lineout tempfile, "<html><head><title>New user </title>"
call lineout tempfile, '</head><body>'
call lineout tempfile,' <strong>You are now logged on as: 'aname'</strong> <pre>'
call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
call lineout tempfile,'<a href="'manname'?mode=FRAMESET%26time='time('s')'" target="_top">ReLoad MANAGER</a>'
call lineout tempfile,' </body> </html> '
call lineout tempfile
return '200 122'


/*************************/
/* does this root (i.e.; virtual directory) contain any allowed directories */
is_allowed_dirtree:procedure expose allowed_dirs. manname rootlist.
parse upper arg adir

ahost=''
if allowed_dirs.0=0 then return 1
if allowed_dirs.!nick.0='' then  do
  ahost='!'||allowed_dirs.!nick
  if allowed_dirs.ahost.0=1 then return 1
end

adir=strip(adir)

if allowed_dirs.0<>'' then do
  do mm=1 to allowed_dirs.0
     if abbrev(allowed_dirs.mm,adir)=1 then return 1  /* there might be a directory in/under this */
  end
end
if ahost='' then return 0       /* not a hostspecific request, nothing more to check. so no match */

if allowed_dirs.ahost.0='' then return 0  /* no entries for this host */
do mm=1 to allowed_dirs.ahost.0
     if abbrev(allowed_dirs.ahost.mm,adir)=1 then return 1  /* there might be a directory in/under this */
end
return 0



/***********************/
/* is this an allowed directory (or subdirectory of allowed directory)? */
is_allowed_dir:procedure expose allowed_dirs. tempfile manname rootlist.
parse upper arg adir,noclose

if allowed_dirs.!checkit<>2 then do  /* compare agains rootlist */
 do mm=1 to words(rootlist.0)
     aroot=strip(word(rootlist.0,mm))
     aroot='$'||aroot
     atry=translate(rootlist.aroot)
     if abbrev(adir'\',atry)=1 then return  1 /* a match */
 end /* do */
 return 0       /* no match */
end

/* else, checkit=2 */
if allowed_dirs.0=0 then return 1       /* default is "always allowed */

AHOST=''
if allowed_dirs.!nick<>'' then do
  ahost='!'||allowed_dirs.!nick
  if allowed_dirs.ahost.0=0 then return 1  /* this host is "always allowed */
end /* do */

adir=strip(adir)
if allowed_dirs.0<>"" then do
 do mm=1 to allowed_dirs.0
   atry=strip(translate(strip(word(allowed_dirs.mm,1))),'t','\')||'\'
   if abbrev(adir'\',atry)=0 then iterate       /* not a match, try next one */
/* if here, a match -- check privileges? */

   if words(allowed_dirs.mm)=1 then return 1   /* no privs, so open to anyone */
   reqprivs=translate(subword(allowed_dirs.mm,2))

   if wordpos('*',reqprivs)>0 then return 1    /* * means "anyone */
   do mm2=1 to words(reqprivs)
       app=strip(word(reqprivs,mm2))
       oof=wordpos(app,allowed_dirs.!privset)
       if oof>0 then return 1
   end /* do */
 end
end


/* try checking host specific entries */
nocc=0
if ahost<>'' then  do
   IF ALLOWED_DIRS.AHOST.0<>'' then nocc=allowed_dirs.ahost.0
end

/* Got some host specific allowed_dirs */
do mm=1 to nocc
       atry=strip(translate(strip(word(allowed_dirs.ahost.mm,1))),'t','\')||'\'
       if abbrev(adir'\',atry)=0 then iterate       
       if words(allowed_dirs.ahost.mm)=1  then return 1   
       reqprivs=translate(subword(allowed_dirs.ahost.mm,2))
       if wordpos('*',reqprivs)>0 then return 1    
       do mm2=1 to words(reqprivs)
         app=strip(word(reqprivs,mm2))
         oof=wordpos(app,allowed_dirs.!privset)
         if oof>0 then return 1
       end 
end

if noclose=1 then return 0              /* if here, no "privileged" match */
call lineout tempfile,'<br><b>Access to this directory is not permittted:'adir'</b></body></html>'
call lineout tempfile
return 0


/***********************/
/* remove or search a directory */
dirdo:
call lineout tempfile,'<html><head><title>SRE-FileManager:Dir Action</title></head>'
call lineout tempfile,'<BODY><h3>Directory Action </h3>'

thedir=translate(varlist.!dir,'\','/')
parse var thedir '$' atype '\' fname
atype=translate(atype)
aa='$'atype
addme=rootlist.aa
if addme='' then do
   call lineout tempfile,' <b>Invalid root directory:</b> 'atype
   call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

dirname=strip(addme,'t','\')||'\'||strip(fname,'l','\')

if is_allowed_dir(dirname)=0 then return 1

if wordpos(varlist.!diraction,'SEARCH REMOVE')=0 then do
   call lineout tempfile,' <b>Invalid action:</b> 'varlist.!diraction
   call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end

if varlist.!diraction="REMOVE" & varlist.!CONFIRM<>1 then do
   foo=sysfiletree(dirname'\*.*','fls','FSO')
   foo=sysfiletree(dirname'\*.*','drs','DSO')

   call lineout tempfile,'<form action="'manname'" method="GET">'
   call lineout tempfile,'<INPUT type="hidden" name="DIRACTION" value="REMOVE">'
   call lineout tempfile,'<INPUT type="hidden" name="DIR" value="'varlist.!dir'">'
   call lineout tempfile,'<input type="hidden" name="MODE" value="DIRDO">'
   call lineout tempfile,'There are 'fls.0 ' files and 'drs.0' subdirectories under <tt>'dirname'</tt><p>'

   call lineout tempfile,'<input type="checkbox" value=1 name="CONFIRM">'
 
   call lineout tempfile,'Are you sure you want to remove: 'dirname
   call lineout tempfile,'<Input type="SUBMIT" value="Yes. Remove this directory">'
   call lineout tempfile,'<br><a href="javascript:self.back()">No. Go Back</a><br>'
   call lineout tempfile,'</form>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

if varlist.!diraction='REMOVE' then do  /* remove this dir (confirm must be 1) */
   foo=sysfiletree(dirname'\*.*','fls','FSO')
   foo=sysfiletree(dirname'\*.*','drs','DSO')
   call lineout tempfile,'Deleting 'fls.0 ' files and 'drs.0' subdirectories under <tt>'dirname'</tt>'
   foo=del_dir(dirname)
   parse var foo a1 a2
   call lineout tempfile,'<p>'a1 ' files and subdirectories deleted ('a2 'failure)'
   call lineout tempfile,'<br><a href="javascript:history.go(-2)">Back to directory tree</a><br>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

/* if here, must be SEARCH */ 
ss=strip(varlist.!string)
dirname2=strip(dirname,'t','\')'\'||strip(ss,'l','\')
oo=sysfiletree(dirname2,'dds','FOS')
call lineout tempfile,' # of matches to 'dirname2' :<b> 'dds.0'</b>'
call lineout tempfile,'<table>'
iadd=length(addme)
do mm=1 to dds.0
   oof='$'||atype||translate(substr(dds.mm,iadd+1),'/','\')
   yfile=filespec('n',dds.mm)
   call lineout tempfile,'<tr><td><a href="'manname'?file='oof'" target="VIEWER"  >'yfile'</a> </td>'
   yogo=filespec('d',dds.mm)||filespec('p',dds.mm)
   oof=translate(substr(yogo,iadd+1),'/','\')
   call lineout tempfile,' <td>  <a href="'manname'?root='atype'%26dir='oof'%26Mode=LIST">'oof'</a>'
   call lineout tempfile,' (<a href="'manname'?root='atype'%26dir='oof'%26Mode=SHORTLIST">s</a>)</td>'
end /* do */
call lineout tempfile,'</table>'

call lineout tempfile,'<br><a href="javascript:self.back()">Back</a><br>'

call lineout tempfile,'</body></html>'
call lineout tempfile
return 1

/**************************/
/* delete a directory tree */
del_dir:procedure expose  allowed_dirs. manname rootlist.
parse arg user_dir

rc=SysFileTree(user_dir,'dir_list', 'D')
IF dir_list.0 = 0  THEN return '0 0'

rc=SysFileTree(user_dir,'dir_list', 'BO', '****','----')

askdel=1
ndid=0 ; nbad=0
rc=SysFileTree(user_dir || '\*.*', 'dir_list', 'FSO')
DO x = 1 TO dir_list.0
 rc = SysFileDelete(dir_list.x)
 if rc=5 & askdel=1 then do      /* set to non readonly */
          if gotch=1 then do
              fop=doschmod(dir_list.x,,'R')
          end /* do */
          rc = SysFileDelete(dir_list.x)
  end
  if rc=0 then
     ndid=ndid+1
  else
    nbad=nbad+1

END

rc=SysFileTree(user_dir || '\*.*', 'dir_list', 'DSO')
DO x = dir_list.0 TO 1 BY -1
 rc=SysRmDir(dir_list.x)
 if rc=0 then
     ndid=ndid+1
 else
     nbad=nbad+1
END

DROP dir_list.
rc=SysRmDir(user_dir)
return ndid' 'nbad


/***********************/
/* deal with uploads */
is_upload:

conttype=reqfield('Content-type')  /* sre-http adodn */

/* IS IT A file upload ? */
if abbrev(upper(conttype),'MULTIPART/FORM-DATA')=0 THEN RETURN 0  /* nope */

nparts=read_multipart_data(list,conttype)  /* parse into form_data. */
varlist.=''
varlist.0=''
do mm=1 to nparts
   aname=translate(form_data.!name.mm) ;aa='!'||aname
   if aname='' then iterate                     /* this should never happen */
   varlist.aa=form_data.mm
   varlist.0=varlist.0' 'aname
   if wordpos('FILENAME',form_data.!list.mm)>0 then do
        aname2=aname'.!ORIGNAME' ; aa='!'aname2
        varlist.aa=form_data.!filename.mm
        varlist.0=varlist.0' 'aname2
   end
end

parse var varlist.!destdir '$' atype '\' fname

atype=translate(atype)
aa='$'atype
addme=rootlist.aa
if addme='' then do 
   call lineout tempfile,'<html><head><title>SRE-FileManager: file upload</title></head>'
   call lineout tempfile,aa' 'rootlist.0' <b>Invalid root directory (upload):</b> 'atype
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

destdir2=addme'\'fname
if is_allowed_dir(destdir2)=0 then return 1

call lineout tempfile,'<html><head><title>SRE-FileManager: file upload</title></head>'
call lineout tempfile,'<BODY><h3>File upload </h3>'
call lineout tempfile,' Uploading to: <tt>'destdir2'</tt><p>'
aa=dir_exists(destdir2)
if aa=0 then do
   call lineout tempfile,'<b> Sorry, no such directory! </b>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

do nn=1 to nparts
   if varlist.!contents.nn='' then iterate
   if varlist.!usename.nn=''  then varlist.!usename.nn=varlist.!contents.nn.!origname
   if varlist.!usename.nn='' then iterate
   toname=filespec('n',varlist.!usename.nn)
   if toname='' then iterate
   call lineout tempfile,' Creating file: <b>'toname '</b>,'
   call lineout tempfile,' w/length of 'length(varlist.!contents.nn)' bytes <p>'
 
   tryname=strip(destdir2,'t','\')'\'toname
   if validname(tryname)<>1 then do
        call lineout tempfile,' <br><b>Error: invalid name: 'tryname'<br>'
        call lineout tempfile,'</body></html>'
        call lineout tempfile
        return 1
   end /* do */

/*now, write it out */
   foo=stream(tryname,'c','open write')
   if abbrev(translate(foo),'READY')<>1 then do
        call lineout tempfile,' <br><b>Error: unable to open for write:'tryname'<br>'
        call lineout tempfile,'</body></html>'
        call lineout tempfile
        return 1
    end
    oobly=charout(tryname,varlist.!contents.nn,1)
    foo=stream(tryname,'c','close')
    if abbrev(translate(foo),'READY')<>1 then do
        call lineout tempfile,' <br><b>Error: unable to complete write:'tryname'<br>'
        call lineout tempfile,'</body></html>'
        call lineout tempfile
        return 1
    end
    call lineout tempfile,' <b> File successfully uploaded </b>'
end

ax='root='atype'&dir=/'fname

call lineout tempfile,'<p><a href="'manname'?'ax'">Reload?</a>'
call lineout tempfile,'</body></html>'
call lineout tempfile
return 1
  
/****************/
/* show a file */
show_file:

istext=0
thefile=translate(varlist.!file,'\','/')
if thefile='' then do
    thefile=translate(varlist.!textfile,'\','/')
   istext=1
end /* do */
parse var thefile '$' atype '\' fname
atype=translate(atype)
aa='$'atype
addme=rootlist.aa
if addme<>'' then do
  thefile=addme'\'fname
end

if is_allowed_dir(thefile)=0 then return 1

foo=''
if addme<>'' then foo=stream(thefile,'c','query exists')

if foo='' then do
  call lineout tempfile,'<html><head><title>SRE-FileManager: no such file </title></head>'
  call lineout tempfile,'<BODY><h3>File not found </h3>'
  call lineout tempfile,'Sorry, could not find: 'varlist.!file
  call lineout tempfile,'</body></html>'
  call lineout tempfile
  iff=stream(tempfile,'c','query size')
  foo=sref_gos('FILE ERASE type text/html name 'tempfile)
  return '200 'iff
end

atype='text/plain'
if istext<>1 then   atype=sref_mediatype(thefile)
foo=sref_gos('FILE nocache type 'atype 'name 'thefile)
iff=stream(thefile,'c','query size')
return '200 'iff


/****************/
/* do the actual DEL, MOVE, COPY */
do_actions:

DELRCtext.0='Error deleting.'
DELRCText.0 = 'File deleted successfully. '
DELRCText.2 = 'Error.  File not found. '
DELRCText.3 = 'Error.  Path not found. '
DELRCText.5 = 'Error.  Access denied. '
DELRCText.26 = 'Error.  Not DOS disk. '
DELRCText.32 = 'Error.  Sharing violation. '
DELRCText.36 = 'Error.  Sharing buffer exceeded. '
DELRCText.87 = 'Error.  Invalid parameter. '
DELRCText.206 = 'Error.  Filename exceeds range error. '

copyrctext.='Error copying.'
COPYRCtext.0 ='File copied successfully'
COPYRCtext.2 ='Error: source file not found '
COPYRCtext.3 ='Error: source or target path not found '
COPYRCtext.5 ='Error: target file exists but A or R mode not specified '
COPYRCtext.32 ='Error: sharing violation for source or target file '
COPYRCtext.108 ='Error: source or target drive locked '
COPYRCtext.112 ='Error: disk full '
COPYRCtext.206 ='Error: invalid source or target file name '
COPYRCtext.267 ='Error: source name is a directory '
COPYRCtext.282='Error: extended attributes not supported for target '
 

moverctext.1='File moved successfully'
moverctext.0='Error moving file.'
moverctext.2='Error: can not move file to a different drive '
moverctext.3='Error: target directory does not exist'
moverctext.4='Error: file already exists'

doover='N'
if varlist.!overwrite=1 then doover='R'

downloadlist.0=0

call lineout tempfile,'<body><h2>Processing .... </h2>'
call lineout tempfile,'<pre>'
do mm=1 to actions.0
    todo=actions.mm.!do
    thefile=actions.mm.!file
    parse var thefile '$' atype '\' fname
    atype=translate(atype)
    aa='$'atype
    addme=rootlist.aa
    if addme='' then iterate

    thefile=addme'\'fname
    if is_allowed_dir(thefile)=0 then return 1

    dest=actions.mm.!dest

    if todo='DEL' then do
       foo=sysfiledelete(thefile)
       call lineout tempfile,'DEL 'thefile': 'delrctext.foo
    end /* do */
    if todo='COPY' then do
       if is_allowed_dir(dest)=0 then return 1
       foo=doscopy(thefile,dest,doover)
       call lineout tempfile,'COPY 'thefile' to 'dest': 'copyrctext.foo
    end
    if todo='DOWNLOAD'  then do
      ndowns=downloadlist.0+1
      downloadlist.ndowns=actions.mm.!file
      downloadlist.0=ndowns
      call lineout tempfile,'DOWNLOAD... 'thefile
    end
    if todo='MOVE' then do
       ug=is_allowed_dir(dest)
       if ug=0 then return 1
       adr1=translate(filespec('d',thefile))
       adr2=translate(filespec('d',dest))
       foo=1
       if adr1<>adr2 then do
           foo=2
       end /* do */
       if foo=1 then do
          aa=dir_exists(dest)
          if aa=0 then foo=3
       end 
       if foo=1 & pos('.',dest)=0 then do
           nm=filespec('n',thefile)
           dest=strip(dest,'t','\')'\'nm
       end 
       if stream(dest,'c','query exists')<>'' then  foo=4
       if foo=1 then  foo=dosrename(thefile,dest)
       if foo=1 then
         call lineout tempfile,'MOVE 'thefile' to 'dest ': 'moverctext.foo
      else
         call lineout tempfile,'MOVE 'thefile 'to ' dest ': 'moverctext.foo ' ('dest')'
    end /* do */
end /* do */
call lineout tempfile,'</pre>'
call lineout tempfile,'<a href="'manname'?" target="_top">ReLoad Manager</a><p>'



if downloadlist.0>0 then do 
   call lineout tempfile,'<hr width="30%"><h3 align="center">'downloadlist.0 " files will be downloaded</h3> "
   call lineout tempfile,'<form action="'manname'" method="POST">'
   call lineout tempfile,'<input type="hidden" name="MODE" value="DOWNLOADS">'
   hh=''
   do iu=1 to downloadlist.0
      hh=hh' 'downloadlist.iu
   end /* do */
   call lineout tempfile,' <input type="hidden" name="FILELIST" value="'hh'">'
   call lineout tempfile,'<table><tr><td>'
   call lineout tempfile,'<select name="DOWNLOAD_TYPE" size=3>'
   call lineout tempfile,'<option value=1 selected>Download as SRE-multipart'
   call lineout tempfile,'<Option value=2>Download as multipart/mixed -- mimetype'
   call lineout tempfile,'<Option value=3>Download as multipart/mixed -- binary'

   call lineout tempfile,'</select> </td><td>'
   call lineout tempfile,'Note that SRE-multipart requires the <a href="DOMULTI.CMD">DoMulti.CMD</a> browser callable application'
   call lineout tempfile,'</td><tr><td>Delay=<input type="text" name="DELAY" value=5 size=5> '
   call lineout tempfile,'<br><font size="-1">(only used with multipart/mixed -- mimetype)</font></td>'
   call lineout tempfile,'<td><input type="SUBMIT" value="Start the Download">'
   call lineout tempfile,'</td></tr></table>'
   call lineout tempfile,'</FORM>'
end
call lineout tempfile,'</body></html>'
call lineout tempfile

return 1


/****************/
/* process input -- this is called upon first submission of del/move/copy choices */
make_doit1:

ACTIONS.=0
actions.0=0
ndownloads=0
/* write file list */
call lineout tempfile,'<html><head><title>SRE-FileManager </title></head>'
call lineout tempfile,'<body><h2>Selected actions </h2>'
call lineout tempfile,'<ul>'
defdest=strip(translate(varlist.!defdest))
if defdest<>'' then do
   isd=dir_exists(defdest)
   if isd=0 then do                     /* error to client */
       'string ERROR: no such default destination directory: 'defdest
       return '200 51'
   end /* do */
end 

do mmm=1 to varlist.!file.0
   if varlist.!DO.mmm='' then iterate
   destd=strip(varlist.!DEST.mmM)
   select
      when destd=0 then iterate      /* 0 means " do nothing "*/
      when destd='' then do             /* use defdest */
         destd=defdest
      end /* do */
      when defdest='' then nop          /* use as is */
      otherwise  do                       /* perhaps append defdest to destd? */
         if pos(':',destd)+pos('\',destd)=0 then do 
            destd=strip(defdest,'t','\')||'\'||destd
         end /* do */
      end
    end

   if abbrev(varlist.!file.mmm,'$')=0 then iterate /* ONLY allow "relative" directories */
   parse var varlist.!file.mmm '$' atype '\' fname
   atype=translate(atype)
   aa='$'atype
   addme=rootlist.aa
   if addme='' then iterate

   fuse=addme'\'fname
   if is_allowed_dir(fuse)=0 then return 0

   if pos(':',destd)+pos("\",destd)=0 then do  /* use current directory */
       destd=filespec('d',fuse)||filespec('p',fuse)||destd
   end

/* may want to do is_allowed_dir on destd */
   if destd<>'' then do
       isd=dir_exists(destd)

       if isd=0 then do
          'string ERROR: no such destination directory 'destd '(for 'fuse')'
          return '200 67'
       end
    end
    doit=strip(translate(varlist.!do.mmm))
    call lineout tempfile,'<li> 'doit' <tt>'fuse'</tt>'
    if doit='COPY' | doit="MOVE" then do
         if destd='' then do
             'string Error: destination required for 'doit' 'fuse
         end /* do */
         call lineout tempfile,' to ' destd
    end /* do */
    if doit='DOWNLOAD' then do
         ndownloads=ndownloads+1
    end /* do */
    IF DOIT='DEL' then DO
       IF STREAM(fuse,'C','QUERY EXISTS')='' then DO
            'string ERROR: No such file 'fuse
             return '200 33'
       end /* do */
       destd=''
    end /* do */
    IAA=ACTIONS.0+1
    ACTIONS.IAA.!DO=DOIT
    ACTIONS.IAA.!FILE=VARLIST.!FILE.MMM
    ACTIONS.IAA.!DEST=DESTD
    ACTIONS.0=IAA
end
call lineout tempfile,'</ul>'
IF VARLIST.!VERIFY=1 then DO
 call lineout tempfile,'<form method="POST" action="'manname'">'
 call lineout tempfile,'<input type="hidden" name="MODE" value="DOIT2">'
 if varlist.!overwrite=1 then
   call lineout tempfile,'<input type="hidden" name="OVERWRITE" value=1>'
 call lineout tempfile,'<input type="hidden" name="ACTIONS.0" value='actions.0'>'
 do mm=1 to actions.0
   call lineout tempfile,'<input type="hidden" name="ACTIONS.'mm'.!DO" value="'actions.mm.!do'">'
   call lineout tempfile,'<input type="hidden" name="ACTIONS.'mm'.!FILE" value="'actions.mm.!file'">'
   call lineout tempfile,'<input type="hidden" name="ACTIONS.'mm'.!DEST" value="'actions.mm.!dest'">'
 end 
 call lineout tempfile,'<input type="hidden" name="ndownloads" value='ndownloads'>'
 call lineout tempfile,'<input type="submit" value="This is correct ">'
 call lineout tempfile,'<b>If this is <b>not</b> correct, <a href="'manname'?" target="_top">ReLoad Manager </a>'
 call lineout tempfile,'</form>'
 call lineout tempfile,'</body></html>'
 call lineout tempfile
 return 0
end

return 1

/********************/
/* convert varlist.!actions... to actions. 
  This is called after a "verification submisstion" */
cvt_actions:
actions.0=varlist.!actions.0
actions.!destdir=varlist.!destdir
do mm=1 to varlist.!actions.0
   actions.mm.!DO=varlist.!actions.mm.!DO
   actions.mm.!FILE=varlist.!actions.mm.!FILE
   actions.mm.!DEST=varlist.!actions.mm.!DEST
end
return 


/****************/
/* show directory tree; allow choice */
make_Tree:

call lineout tempfile,'<html><head><title>SRE-FileManager:Dir Tree </title></head>'
call lineout tempfile,'<body><h3 align="Center">Directory Tree</h3><p>'

do jj=1 to words(rootlist.0)
   aroot=strip(word(rootlist.0,jj))
   call lineout tempfile,'<a href="#TO_'AROOT'">'aroot'</a>||'
   if (jj//6)=0 then call lineout tempfile,'<br>'
end /* do */
call lineout tempfile,"<br>"

call lineout tempfile,'<FORM name="DIR_DO" action="'manname'" method="POST">'
call lineout tempfile,'<input type="HIDDEN" name="MODE" value="DIRDO">'
call lineout tempfile,'<table><tr><td>Choose an action: <br>'
call lineout tempfile,'<select size=3 name="DIRACTION">'
call lineout tempfile,'<option value="none" selected> ---- '
call lineout tempfile,'<option value="REMOVE">Remove Directory'
call lineout tempfile,'<option value="SEARCH">Search for a file'
call lineout tempfile,'</select>'
call lineout tempfile,'</td><td>Search for: <input type="text" name="STRING" size=40> </td>'
call lineout tempfile,'</table>'
call lineout tempfile,'<center><Input type="submit" value="...process"></center><br>'


do jj=1 to words(rootlist.0)
  aroot=strip(word(rootlist.0,jj))
  aa='$'aroot
  tdir=rootlist.aa
   
  if is_allowed_dirtree(tdir)=0 then iterate /* no allowed_dirs under this? */
/* else, at least one allowed_dirs in/under this tdir */
  aa=sysfiletree(tdir'\*.*','dlist','DoS')

  call lineout tempfile,'<a name="TO_'aroot'"><hr width="66%"> </a>'

  call lineout tempfile,'<font size="+1">There are 'dlist.0' subdirectories under 'tdir'</font>,'
  drop dlist2.
  do mm=1 to dlist.0
     adir=dlist.mm
     dlist2.mm=' '
     if is_allowed_dir(adir,1)=0 then iterate
     adir=translate(substr(adir,length(tdir)+2))
     dlist2.mm=adir
     dlist2.mm=translate(adir,'01'x,'\')
  end
  dlist2.0=dlist.0
  if dlist2.0>0 then  foo=arraysort(dlist2) 
  do mm=1 to dlist2.0
     dlist2.mm=translate(dlist2.mm,'\','01'x)
  end
  foo=make_tree2(tdir,aroot,'root of 'aroot)
end
call lineout tempfile,'<INPUT type="SUBMIT" value="Process ..."></form>'
call lineout tempfile,'</body></html>'
call lineout tempfile

return 1


/*******************/
/* table of dir entries */
make_Tree2:procedure expose dlist2. tempfile allowed_dirs. tdir manname rootlist.
parse arg realroot,aroot,amess

call lineout tempfile,'<table>'

aba='<input type="radio" name="DIR" value="$'aroot||'/">'
if is_allowed_dir(tdir,1)=1 then 
   call lineout tempfile,'<tr><td>'aba' </td><td><a href="'manname'?root='aroot'%26dir=/"  target="main_frame"> <font size="+1">/ </font></a></td><td>('amess') <font size="-1"> </td>'
else
   call lineout tempfile,'<tr><td> ~  </td><td><font size="+1">/ </font></a></td><td>('amess') <font size="-1"> </td>'

do mm=1 to dlist2.0
   adir=dlist2.mm

   if adir='' then iterate
   adir00=realroot'\'adir
   adir0=translate(adir,'/','\')
   aba='<input type="radio" name="DIR" value="$'aroot||'/'||adir0'">'

   if pos('\',adir)=0 then
     call lineout tempfile,'<tr><td>'aba' </td><td><hr noshade size=1 color="#445566"> </td>'
   else
     call lineout tempfile,'<tr><td>'aba' </td><td> </td>'

   do forever
      parse var adir a1 '\' adir
      if adir<>'' then do
         if pos('\',adir)=0 then
            call lineout tempfile,'<td><hr noshade size=1 color="#445566"></td>'
         else
            call lineout tempfile,'<td></td>'
      end /* do */
      else do
         lrefs=' <a href="'manname'?mode=SHORTLIST%26root='aroot'%26dir=/'adir0'" target="main_frame">[c]</a> '
         lref='<a href="'manname'?root='aroot'%26dir=/'adir0'" target="main_frame"> 'a1'</a>'
        
         chk=adir00'\*.*'
         oink=sysfiletree(chk,'gg','OF')
          call lineout tempfile,'<td>'lref||lrefs'<em>('gg.0')</em></td></tr>'
          leave
      end
   end
end /* do */
call lineout tempfile,'</table>'
return 0


/****************/
/* show directory tree; allow choice */
make_Tree_compact:

call lineout tempfile,'<html><head><title>SRE-FileManager: Show Compact Dir </title></head>'
call lineout tempfile,'<body><b>Directory Tree:</b>'
call lineout tempfile,'<font size="-1">'servername'<br>'

do jj=1 to words(rootlist.0) 
   aroot=strip(word(rootlist.0,jj))
   aa='$'aroot
   tdir=rootlist.aa

  if is_allowed_dirtree(tdir)=0 then iterate /* no allowed_dirs under this? */

/* else, at least one allowed_dirs in/under this tdir */
   aa=sysfiletree(tdir'\*.*','dlist','DoS')

   do mm=1 to dlist.0
      adir=dlist.mm
      dlist2.mm=' '
      if is_allowed_dir(adir,1)=0 then iterate
      adir=translate(substr(adir,length(tdir)+2))
      dlist2.mm=translate(adir,'01'x,'\')
  end
  dlist2.0=dlist.0

  if dlist2.0>0 then foo=arraysort(dlist2) 

  do mm=1 to dlist2.0
     dlist2.mm=translate(dlist2.mm,'\','01'x)
  end /* do */
  foo=make_tree_compact2(aroot,aroot' directory')

  call lineout tempfile,'<hr width="50%">'
end
call lineout tempfile,'</font></body></html>'
call lineout tempfile

return 1


/***************/
/* display a "compact" directory list, of the dlist2. stem */
make_tree_compact2:procedure expose dlist2. tempfile  allowed_dirs. tdir manname rootlist.
parse arg damode,amess

call lineout tempfile,'<pre>'

if is_allowed_dir(tdir,1)=1 then 
   call lineout tempfile,'<a href="'manname'?root='damode'%26dir=/" target="main_frame">/ </a> ('amess')'
else 
   call lineout tempfile,' /  ('amess')'

do mm=1 to dlist2.0
   adir=dlist2.mm
   if adir='' then iterate
   adir0=translate(adir,'/','\')

   if pos('\',adir)=0 then
     alin=' __'
   else
     alin='   '
   do forever
      parse var adir a1 '\' adir
      if adir<>'' then do
         if pos('\',adir)=0 then
            alin=alin||'__'
         else
           alin=alin||'  '
      end /* do */
      else do
         lrefs=' <a href="'manname'?mode=SHORTLIST%26root='damode'%26dir=/'adir0'" target="main_frame">[c]</a> '
         lref=' <a href="'manname'?root='damode'%26dir=/'adir0'" target="main_frame">'a1'</a>'
         alin=alin||' 'lref||lrefs
         call lineout tempfile,alin
         leave
      end
   end
end /* do */
   
call lineout tempfile,'</pre>'
return 1


/****************/
/* make a control panel frame */
make_a_control:
crlf='0d0a'x

foo=sref_extract_upwd()
parse var foo aname ':' . ; aname=strip(aname)

aout=' onMouseOut="window.status=''The SRE-FileManager Control Panel'' ; return true " '

call lineout tempfile,'<html><head><title>SRE-FileManager: control panel </title>'
call lineout tempfile,'<script language="javascript">'
call lineout tempfile,'function Open_viewtree(isblur)'
call lineout tempfile,'{daVIEWTREE=window.open("'manname'?mode=TREECOMPACT","VIEWTREE",',
                     '"location=NO,MENUBAR=NO,RESIZABLE,DEPENDENT,SCROLLBARS,STATUS=NO,TITLEBAR,TOOLBAR=NO,SCREENX=12,SCREENY=30,height=440,width=200")'
call lineout tempfile,' var openVIEWTREE=1  '
call lineout tempfile, 'if (isblur==1) ; {daVIEWTREE.blur() }'
call lineout tempfile,' }'                    
call lineout tempfile,' '
call lineout tempfile,'function dolist(ltype,ii) {'
call lineout tempfile,'adir=parent.main_frame.viewing'
call lineout tempfile,'rot=parent.main_frame.rootis'
call lineout tempfile,'if (ii==1)'
call lineout tempfile,'{gto="'manname'?mode=TREECOMPACT"'
call lineout tempfile,'  daVIEWTREE.location=gto}'
call lineout tempfile,' else {'
call lineout tempfile,'gto="'manname'?root="+rot+"%26mode="+ltype+"%26dir="+adir'
call lineout tempfile,'gto2="http://"+window.location.hostname+gto'
call lineout tempfile,'top.main_frame.location.href=gto2 }'
call lineout tempfile,'}'
call lineout tempfile,'Open_viewtree(1) '
call lineout tempfile,'</script>'

call lineout tempfile,'</head>'
call lineout tempfile,'<body><nobr><b>Control Panel</b></nobr><br>'
call lineout tempfile,'<font size="-1"> Key: </font><br>'
call lineout tempfile,'<table><tr><td bgcolor="'delcolor'"><font size="-1">Del</font></td>' 
call lineout tempfile,'<td bgcolor="'movecolor'"><font size="-1">Move</font></td>'
call lineout tempfile,'<td bgcolor="'copycolor'"><font size="-1">Copy</font></td>'
call lineout tempfile,'<td bgcolor="'downloadcolor'"><font size="-1">Down</font></td></table></font><br>'


astatus=aout||' onMouseOver="window.status='||"'"||' ... make the changes '||"'"||';return true "'
call lineout tempfile,'<a href="javascript:parent.main_frame.location.hash=''DOSUBMIT2''; void(0)"'||astatus||' >Submit!</a><br>'

astatus=' onMouseOver="window.status='||"'"||'[current user: 'aname'] :: Logon as a different user (you may need to first reload this frame)'||"'"||';return true "'
call lineout tempfile,'<a href="'manname'?mode=logon%26user='aname'" target="main_frame" 'astatus' >Logon</a><p>'

onc=' onCLick="Open_viewtree(); daVIEWTREE.focus() ; return true "'
astatus=onc||aout||' onMouseOver="window.status='||"'"||'Display compact directory in separate window'||"'"||' ;return true"'
call lineout tempfile,'<nobr><a href="javascript:dolist('||"'"||'TREECOMPACT'||"'"||',1) ; void(0)"'
call lineout tempfile,astatus||'>[c]</a>'

astatus=aout||' onMouseOver="window.status='||"'"||'Display directory in main frame'||"'"||' ;return true "'
call lineout tempfile,'<a href="'manname'?mode=TREE" target="main_frame"||'astatus||'>DirTree</a></nobr><p>'

astatus=aout||' onMouseOver="window.status='||"'"||'Jump to file section of main frame'||"'"||' ;return true "'
call lineout tempfile,'<a href="javascript:parent.main_frame.location.hash=''FILELIST''; void(0)"'||astatus||'>Files</a><br>'

astatus=aout||' onMouseOver="window.status='||"'"||'Jump to subdirectories section of main frame'||"'"||' ;return true "'
call lineout tempfile,'<a href="javascript:parent.main_frame.location.hash=''DIRLIST''; void(0)"'||astatus||' >SubDirectories</a><br>'

astatus=aout||' onMouseOver="window.status='||"'"||'Jump to file upload section of main frame'||"'"||' ;return true "'
call lineout tempfile,'<a href="javascript:parent.main_frame.location.hash=''UPLOADS''; void(0)"'||astatus||' >Upload</a><p>'

goose="var tmp1='List of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,'<a href="javascript:dolist('||"'"||'LIST'||"'"||') ; void(0)"'||astatus||'>List</a> &nbsp;'

goose="var tmp1='Extension sorted list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,' (<a href="javascript:dolist('||"'"||'EXT_LIST'||"'"||') ; void(0)"'||astatus||'>e</a>'

goose="var tmp1='Date sorted list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,' <a href="javascript:dolist('||"'"||'DATE_LIST'||"'"||') ; void(0)"'||astatus||'>d</a>'

goose="var tmp1='Name sorted list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,' <a href="javascript:dolist('||"'"||'NAME_LIST'||"'"||') ; void(0)"'||astatus||'>n</a>)<br>'

goose="var tmp1='Compact list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,'<nobr><a href="javascript:dolist('||"'"||'SHORTLIST'||"'"||') ; void(0)"'||astatus||'>QuickList</a>&nbsp;'

goose="var tmp1='Extension sorted, compact list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,' (<a href="javascript:dolist('||"'"||'EXT_SHORTLIST'||"'"||') ; void(0)"'||astatus||'>e</a>'

goose="var tmp1='Date sorted, compact list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,' <a href="javascript:dolist('||"'"||'EXT_DATE_LIST'||"'"||') ; void(0)"'||astatus||'>d</a>'

goose="var tmp1='Name sorted, compact list of files in:' + parent.main_frame.viewing ; window.status=tmp1 ;return true  "
astatus=aout||' onMouseOver="'||goose||'"'
call lineout tempfile,' <a href="javascript:dolist('||"'"||'EXT_NAME_LIST'||"'"||') ; void(0)"'||astatus||'>n</a>)<br>'


call lineout tempfile,'<p><a href="manager.doc" target="VIEWER">Documentation</a>'

call lineout tempfile,'</body></html>'
call lineout tempfile

return 0



/****************/
/* make the frameset */
make_frames:

call lineout tempfile,'<html><head><title>The SRE-FileManager </title></head>'
call lineout tempfile,'<frameset name="THEMANAGER" cols="15%,*" >'
    call lineout tempfile,'  <frame src="'manname'?mode=toc" name="toc_frame">'
    call lineout tempfile,'  <frame src="'manname'?mode=list&dir=/" name="main_frame">'
call lineout tempfile,'<noframes>'
call lineout tempfile,'This is the <b>frame</b> version of the SRE-http file manager.'
call lineout tempfile,'If you are seeing this message, your browser does not'
call lineout tempfile,'support frames -- try the <a href="'manname'?mode=list" target="_top"> non-frames version </a>'
call lineout tempfile,'</noframes>'
call lineout tempfile,'</frameset>'
call lineout tempfile,'</body></html>'
call lineout tempfile
return 0


/***********************/
/* write a del/move/copy enabled list of files and directories */
make_list:
parse arg sortext

/* write file list */
call lineout tempfile,'<html><head><title>SRE-FileManager </title>'
call lineout tempfile,'<script language="javascript">'
vvar=varlist.!dir
if vvar='' then vvar='/'
call lineout tempfile,'var viewing="'vvar'"'
call lineout tempfile,'var rootis="'using_root_alias'"'


call lineout tempfile,'</script>'
call lineout tempfile,'</head>'
call lineout tempfile,'<body><h2>SRE-FileManager: display directory </h2>'

if is_allowed_dir(ddir)=0 then return 1


call lineout tempfile,'<form name="DO_MANAGER" method="POST" action="'manname'">'
call lineout tempfile,'<input type="hidden" value="DOIT" name="mode">'
call lineout tempfile,'<input type="hidden" name="FILE.0" value="'ddirfiles.0'">'

call lineout tempfile,'<font size="+2"><b><a name="FILELIST">'ddirfiles.0' files in 'ddir'</a></b></font>,'
call lineout tempfile,' and 'ddirdirs.0' <a href="#DIRLIST"> subdirectories</a> <p>'

call lineout tempfile,' '

call lineout tempfile,'<table border=2> <tr><td><input type="checkbox" name="VERIFY" value=1 checked>Verify choices?</td>'
call lineout tempfile,'<td><input type="checkbox" name="OVERWRITE" value=1 checked>Overwrite allowed?</td>'
call lineout tempfile,'<td><nobr>Default destination-directory:</nobr></a><input type="text" name="DEFDEST" size=20></td>'
call lineout tempfile,'<tr><td colspan=2><input type="checkbox" name="EDIT" value=1> Edit file (the first file with a checked COPY button)?</td>'
call lineout tempfile,'<td><input type="reset" value="reset"> '
call lineout tempfile,'<a name="DOSUBMIT2"><input type="submit" value="make the changes"></a><br></td></table>'

call lineout tempfile,' '
call lineout tempfile,'<table padding=0><tr><td>'
call lineout tempfile,'<table width="100%"><tr><td bgcolor="'delcolor'">Del</td>' 
call lineout tempfile,'<td bgcolor="'movecolor'">Move</td>'
call lineout tempfile,'<td bgcolor="'copycolor'">Copy</td>'
call lineout tempfile,'<td bgcolor="'downloadcolor'">Down</td></table></td>'

call lineout tempfile,'<td>Name</td> <td width="10%">Date &amp; Size</td> <td bgcolor="#5aedf9" align="left">Destination directory (used with MOVE &amp; COPY) <br><b>0</b>: <em>no change</em><br>blank=<em>use default</em> </td>'
call lineout tempfile,'<tr> <td colspan=4><hr></td>'

if sortext=1 then do
   do mm=1 to ddirfiles.0
      ff=lastpos('.',ddirfiles.mm)
      if ff=0 then 
         aext=''
      else
         aext=translate(substr(ddirfiles.mm,ff+1))
      ext=translate(left(aext,20))
      ddirfiles.mm=ext||ddirfiles.mm
   end /* do */
   if ddirfiles.0>0 then foo=arraysort(ddirfiles)
   do mm=1 to ddirfiles.0
       ddirfiles.mm=substr(ddirfiles.mm,21)
   end /* do */
end /* do */

if sortext=2 then do
   do mm=1 to ddirfiles.0
      ff=strip(word(ddirfiles.mm,1))
      if left(ff,2)<70 then
        ff='20'||ff
      else
        ff='19'||ff
      ff=translate(left(ff,20))
      ddirfiles.mm=ff||ddirfiles.mm
   end /* do */
   if ddirfiles.0>0 then foo=arraysort(ddirfiles)
   do mm=1 to ddirfiles.0
       ddirfiles.mm=substr(ddirfiles.mm,21)
   end /* do */
end /* do */

if sortext=3 then do
   do mm=1 to ddirfiles.0
      ff=strip(word(ddirfiles.mm,words(ddirfiles.mm)))
      ff=filespec('n',ff)
      parse var ff ff '.' .  
      ff=translate(left(ff,20))
      ddirfiles.mm=ff||ddirfiles.mm
   end /* do */
   if ddirfiles.0>0 then foo=arraysort(ddirfiles)
   do mm=1 to ddirfiles.0
       ddirfiles.mm=substr(ddirfiles.mm,21)
   end /* do */
end /* do */



do mm=1 to ddirfiles.0
    aa=make_file_entry(ddirfiles.mm,mm,using_root,using_root_alias)
    call lineout tempfile,aa
end
call lineout tempfile,'</table>'

call lineout tempfile,'<a name="DIRLIST"><hr width="20%"></a>'
/*
call lineout tempfile,'<br><input type="reset" value="reset"> &nbsp;<input type="submit" value="make the changes">'
*/

call lineout tempfile,'<h3> <a name="DIRLIST2">' ddirdirs.0 ' subDirectories under 'ddir'</a></h3>'
call lineout tempfile,'<ul>'

if using_root<>ddir then do
  aa=make_dir_entry('..',ddir,using_root)
  call lineout tempfile,aa
end

do mm=1 to ddirdirs.0
   aa=make_dir_entry(ddirdirs.mm,using_root)
   call lineout tempfile,aa
end
call lineout tempfile,'</ul>'
call lineout tempfile,'</form>'
foof='$'||using_root_alias||substr(ddir,length(using_root)+1)

call lineout tempfile,'<FORM name="BFORM" enctype="multipart/form-data" ACTION="'manname'" METHOD="post" >'
call lineout tempfile,'<input type="hidden" name="destdir" value="'foof'">'
call lineout tempfile,'<a name="UPLOADS"><hr width="73%"></a>'
call lineout tempfile,'<h3> Upload a file to: 'ddir'</h3>'
call lineout tempfile,'File to upload:<INPUT TYPE="file"  name="CONTENTS.1"><br>'
call lineout tempfile,'<br>Use name: <input type="text" name="usename.1"><em>leave empty to use "own name"</em>'
call lineout tempfile,''
call lineout tempfile,'<br><input type="reset" value="reset"> &nbsp;<input type="submit" value="Upload the file">'
call lineout tempfile,'</form>'

call lineout tempfile,'</form></body></html>'
call lineout tempfile

return 0



/***********************/
/* write a short del/move/copy enabled list of files and directorie */
make_short_list:
parse arg sortext

if is_allowed_dir(ddir)=0 then return 1


/* write file list */
call lineout tempfile,'<html><head><title>SRE-FileManager: display director (condensed) </title>'
call lineout tempfile,'<script language="javascript">'
vvar=varlist.!dir
if vvar='' then vvar='/'
call lineout tempfile,'var viewing="'vvar'"'
call lineout tempfile,'var rootis="'using_root_alias'"'

call lineout tempfile,'</script>'
call lineout tempfile,'</head>'

call lineout tempfile,'<body><h2>SRE-FileManager: display directory (condensed) </h2>'
call lineout tempfile,'<form name="DO_MANAGER" method="POST" action="'manname'">'
call lineout tempfile,'<input type="hidden" value="DOIT" name="mode">'
call lineout tempfile,'<input type="hidden" name="FILE.0" value="'ddirfiles.0'">'

call lineout tempfile,'<font size="+2"><b><a name="FILELIST">'ddirfiles.0' files in 'ddir'</a></b></font>,'
call lineout tempfile,' and 'ddirdirs.0' <a href="#DIRLIST"> subdirectories</a> <p>'


call lineout tempfile,'<table><tr><td><input type="checkbox" name="VERIFY" value=1 checked>Verify choices?</td>'
call lineout tempfile,'<td><input type="checkbox" name="OVERWRITE" value=1 checked>Overwrite allowed?</td>'
call lineout tempfile,'<td><nobr>Default destination-directory:</nobr></a><input type="text" name="DEFDEST" size=20></td></table>'
call lineout tempfile,' '
call lineout tempfile,'<table align="center"><tr> '
call lineout tempfile,'<td><input type="reset" value="reset"></td>'

call lineout tempfile,'<td><table border=1 align="center"><tr align="center"><Td>Key:</td><td> <table><tr>'
call lineout tempfile,'<td bgcolor="'delcolor'">Del</td>' 
call lineout tempfile,'<td bgcolor="'movecolor'">Move</td>'
call lineout tempfile,'<td bgcolor="'copycolor'">Copy</td>'

call lineout tempfile,'<td> <u>FileName</u> </td></tr></table></td>'
call lineout tempfile,'</td></tr></table>'
call lineout tempfile,'<td><a name="DOSUBMIT2"><input type="submit" value="make the changes"></a></td></tr></table>'

if sortext=1 then do
   do mm=1 to ddirfiles.0
      ff=lastpos('.',ddirfiles.mm)
      if ff=0 then 
         aext=''
      else
         aext=translate(substr(ddirfiles.mm,ff+1))
      ext=translate(left(aext,20))
      ddirfiles.mm=ext||ddirfiles.mm
   end /* do */
   if ddirfiles.0>0 then foo=arraysort(ddirfiles)
   do mm=1 to ddirfiles.0
       ddirfiles.mm=substr(ddirfiles.mm,21)
   end /* do */
end /* do */

if sortext=2 then do
   do mm=1 to ddirfiles.0
      ff=strip(word(ddirfiles.mm,1))
      if left(ff,2)<70 then
        ff='20'||ff
      else
        ff='19'||ff
      ff=translate(left(ff,20))
      ddirfiles.mm=ff||ddirfiles.mm
   end /* do */
   if ddirfiles.0>0 then foo=arraysort(ddirfiles)
   do mm=1 to ddirfiles.0
       ddirfiles.mm=substr(ddirfiles.mm,21)
   end /* do */
end /* do */

if sortext=3 then do
   do mm=1 to ddirfiles.0
      ff=strip(word(ddirfiles.mm,words(ddirfiles.mm)))
      ff=filespec('n',ff)
      parse var ff ff '.' .  
      ff=translate(left(ff,20))
      ddirfiles.mm=ff||ddirfiles.mm
   end /* do */
   if ddirfiles.0>0 then foo=arraysort(ddirfiles)
   do mm=1 to ddirfiles.0
       ddirfiles.mm=substr(ddirfiles.mm,21)
   end /* do */
end /* do */






mxflen=0
do mm=1 to ddirfiles.0
     ill=length(strip(filespec('n',ddirfiles.mm)))
   mxflen=max(mxflen,ill)
end /* do */

kols=3

if mxflen>12 then kols=2

call lineout tempfile,'<table><tr> <td colspan=8><hr></td>'
do mm=1 to ddirfiles.0
    aa=make_SHORT_file_entry(ddirfiles.mm,mm,using_root,using_root_alias,kols)
    call lineout tempfile,aa
end
call lineout tempfile,'</table>'

call lineout tempfile,'<a name="DIRLIST"><hr width="20%"></a>'


call lineout tempfile,'<h3> <a name="DIRLIST2">' ddirdirs.0 ' subDirectories under 'ddir'</a></h3>'
call lineout tempfile,'<table>'


if using_root<>ddir then do
  aa=make_dir_entry_short(0,'..',ddir,using_root)
  call lineout tempfile,aa
end

do mm=1 to ddirdirs.0
   aa=make_dir_entry_short(mm,ddirdirs.mm,using_root)
   call lineout tempfile,aa
end
call lineout tempfile,'</table>'
call lineout tempfile,'</form></body></html>'
call lineout tempfile

return 0


/***********************/
/* parse a sysfileetree dir entry ... */
make_dir_entry:procedure  expose downloadcolor copycolor delcolor movecolor using_root_alias  allowed_dirs. manname rootlist.

parse arg adir,basedir,basedir0
if adir='..' then do  /* special case: parent directory */
 adir=strip(basedir,'t','\')
 foo=lastpos('\',adir)
 if foo=0 then return ''                /* at root of drive */
 adir=left(adir,foo)
 selname=substr(adir,length(basedir0)+1)
  selname=translate(selname,'/','\')
  if pos('&',selname)>0 then selname=sref_replacestrg(selname,'&','%26','ALL')
  if pos('#',selname)>0 then selname=sref_replacestrg(selname,'#','%23','ALL')
  if pos('"',selname)>0 then selname=sref_replacestrg(selname,'&','%22','ALL')

  entry='<li><a href="'manname'?root='using_root_alias'%26dir='selname'">  .. (parent) </a>'
  return entry
end /* do */

/* else, regular dir entry */
selname=substr(adir,length(basedir)+1)
selname=translate(selname,'/','\')
entry='<li><a href="'manname'?root='using_root_alias'%26dir='selname'">'selname'</a>'
return entry

/***********************/
/* parse a sysfileetree dir entry ... */
make_dir_entry_short:procedure  expose downloadcolor copycolor delcolor movecolor using_root_alias  allowed_dirs. manname rootlist.

parse arg mm,adir,basedir,basedir0
if adir='..' then do  /* special case: parent directory */
 adir=strip(basedir,'t','\')
 foo=lastpos('\',adir)
 if foo=0 then return ''                /* at root of drive */
 adir=left(adir,foo)
 selname=substr(adir,length(basedir0)+1)
 if pos('&',selname)>0 then selname=sref_replacestrg(selname,'&','%26','ALL')
 if pos('#',selname)>0 then selname=sref_replacestrg(selname,'#','%23','ALL')
 if pos('"',selname)>0 then selname=sref_replacestrg(selname,'&','%22','ALL')

  selname=translate(selname,'/','\')
  entry='<tr><td> <a href="'manname'?mode=SHORTLIST%26root='using_root_alias'%26dir='selname'">  .. (parent) </a></td></tr>'
  return entry
end /* do */


/* else, regular dir entry */
selname=substr(adir,length(basedir)+1)
selname=translate(selname,'/','\')

ff=lastpos('\',adir)
selnamex=substr(adir,ff+1)

nn=mm//6
select
 when nn=0 then 
  entry='<td>  <a href="'manname'?mode=SHORTLIST%26root='using_root_alias'%26dir='selname'">'selnamex'</a> </td></tr>'
 when nn=1 then 
  entry='<tr><td><a href="'manname'?mode=SHORTLIST%26root='using_root_alias'%26dir='selname'">'selnamex'</a> </td>'
 otherwise
  entry='<td> <a href="'manname'?mode=SHORTLIST%26root='using_root_alias'%26dir='selname'">'selnamex'</a> </td>'
end 
return entry


/***********************/
/* parse a sysfiletree file entry, and make it into a table row with link */
make_file_entry:procedure  expose downloadcolor copycolor delcolor movecolor   allowed_dirs. manname rootlist.

parse arg eafile,nf,basedir,diralias
crlf='0d0a'x
parse var eafile adate  asize . aname
aname=strip(aname)
adate=strip(adate)
foo=left(adate,2)
if foo<78 then
  adate='20'||adate
else
  adate='19'||adate
adate=overlay(' ',adate,11,1)
adate=overlay(':',adate,14,1)

selname=translate(substr(aname,length(basedir)+1),'/','\')
if pos('&',selname)>0 then selname=sref_replacestrg(selname,'&','%26','ALL')
if pos('#',selname)>0 then selname=sref_replacestrg(selname,'#','%23','ALL')
if pos('"',selname)>0 then selname=sref_replacestrg(selname,'&','%22','ALL')

selnamex=manname'?file=$'diralias||selname

selname0=filespec('n',eafile)

asize=insertcommas(asize)
iji=length(asize)
do ii2=iji+1 to 12
  asize='&nbsp;'asize
end /* do */


if right(translate(aname),4)<>'.ZIP' then do
   selnamey=manname'?textfile=$'diralias||selname
   selnamey='<a href="'selnamey'" target="VIEWER"> (txt)</a>'
end
else do   
   selnamey=manname'?zipfile=$'diralias||selname
   selnamey='<a href="'selnamey'" target="main_frame"> (zip)</a>'
end /* do */
entry='<td><a href="'selnamex'" target="VIEWER">'selname0'</a> 'selnamey' </td><td width="10%">'adate' 'asize' </td>'

/* now put in buttons */
aname=strip(aname)
aff='$'diralias||substr(aname,length(basedir)+1)

but0='<input type="hidden" name="FILE.'||nf'" value="'aff'">'
but1='<td><table width="100%"><tr ><td bgcolor="'delcolor'">'||but0||'<input type="radio" value="DEL" name="do.'nf'"></td>'
but2='<td bgcolor="'movecolor'"><input type="radio" value="MOVE" name="do.'nf'"></td>'
but3='<td bgcolor="'copycolor'"> <input type="radio" value="COPY" name="do.'nf'"></td>'
but4='<td bgcolor="'downloadcolor'"> <input type="radio" value="DOWNLOAD" name="do.'nf'"></td></table></td>'

in1='<td bgcolor="#5aedf9"><input type="text" name="DEST.'nf'" size=20></td>'
entry='<tr>'but1||but2||but3||but4||entry||in1
return entry



/***********************/
/* parse a sysfiletree file entry, and make it into a table row with link */
make_SHORT_file_entry:procedure expose downloadcolor copycolor delcolor movecolor using_root   allowed_dirs. manname rootlist.
parse arg eafile,nf,basedir,diralias,kols
crlf='0d0a'x
parse var eafile adate  asize . aname
aname=strip(aname)
adate=strip(adate)
foo=left(adate,2)
if foo<78 then
  adate='20'||adate
else
  adate='19'||adate
adate=overlay(' ',adate,11,1)
adate=overlay(':',adate,14,1)

selname=translate(substr(aname,length(basedir)+1),'/','\')
if pos('&',selname)>0 then selname=sref_replacestrg(selname,'&','%26','ALL')
if pos('#',selname)>0 then selname=sref_replacestrg(selname,'#','%23','ALL')
if pos('"',selname)>0 then selname=sref_replacestrg(selname,'&','%22','ALL')

selnamex=manname'?file=$'diralias||selname

selname0=filespec('n',eafile)

asize=insertcommas(asize)
iji=length(asize)
do ii2=iji+1 to 12
  asize=' 'asize
end /* do */

/* the buttons */
aname=strip(aname)
aff='$'diralias||substr(aname,length(using_root)+1)
a1a='<input type="hidden" name="FILE.'||nf'" value="'aff'">'
a1='<input type="radio" value="DEL"  name="do.'nf'">'
a2='<input type="radio" value="MOVE" name="do.'nf'">'
a3='<input type="radio" value="COPY" name="do.'nf'">'
a4='<input type="radio" value="DOWNLOAD" name="do.'nf'">'

/* the file link */
winstat=' onMouseOver="window.status='||"'"||selname0' 'adate' 'asize||"';return true "||'"'
a4='<a href="'||selnamex||'"'||winstat||'" target="VIEWER">'||selname0||'</a>'

aline='<td><table><tr>'
aline=aline||'<td bgcolor="'delcolor'">'||a1a||a1'</td>'crlf 
aline=aline||'<td bgcolor="'movecolor'">'a2'</td>'crlf
aline=aline||'<td bgcolor="'copycolor'">'a3'</td>'crlf

aline=aline||'<td>'a4'</td>'crlf
aline=aline||'</tr></table></td>'
if (nf//kols)==1 then aline='<tr>'aline
if (nf//kols)==0 then aline=aline'</tr>'crlf

return aline




/************/
/*  parse varlist, return in varlisrt.!name.
   With varlist.0 containing list of names */
read_vars:procedure expose varlist.  allowed_dirs. manname rootlist.

parse arg alist
varlist.=''
alist=translate(alist, ' ', '+'||'090a0d'x)  /* Whitespace, etc. */

do forever
    if alist='' then leave
    parse var alist a1 '&' alist
    parse var a1 aname '=' avalue
    if avalue='' then iterate             /* ignore empty entries */
    aname=packur(translate(space(translate(aname,' ','+'||'00090a0d'x),0)))
    aval=strip(packur(translate(avalue,' ','+'||'00090a0d'x)))
    aa='!'||aname
    varlist.aa=aval
    varlist.0=varlist.0' 'aname
end
return varlist.0




/***************/
/* check for SUPERUSER permissions */
is_allowed:procedure expose who2 saddr2 tempfile privset  allowed_dirs. servername manname rootlist.
parse arg checkit 

NO_REM=VALUE('SREF_NO_REMOTE_CONFIG',,'OS2ENVIRONMENT')
who2=extract('CLIENTADDR')
saddr2=extract('SERVERADDR')
select
   when checkit=1 | (checkit=0 & no_rem>0) then do

/* only if user = serveraddress !!! */
     if (who2<>saddr2 | no_rem=2)  then do

        call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 3.0//EN">'
        call lineout tempfile, "<html><head><title>SRE-http MANAGER: not allowed</title>"
        call lineout tempfile, '</head><body>'

        call lineout tempfile,' <strong> Action not allowed remotely.</strong> <pre>'

        call lineout tempfile,' </body> </html> '
        call lineout tempfile
        foo=sref_gos('FILE ERASE TYPE text/html NAME' tempfile)
        signal off error ; signal off syntax
        return '401 Not Permitted'
     end
   end

   when checkit=2  & reqfield('authorization')='' then do               /* always get user name */
      'header add WWW-Authenticate: Basic Realm=<Manager>'  /* challenge */
        call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 3.0//EN">'
        call lineout tempfile, "<html><head><title>MANAGER. Requires a  username </title>"
        call lineout tempfile, '</head><body bgcolor="#'||usecolor||'">'
        call lineout tempfile,' <strong>To use MANAGER, you must supply a username and password.</strong> <pre>'
        call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
        call lineout tempfile,' </body> </html> '
        call lineout tempfile
        signal off error ; signal off syntax
        return sref_response('unauth'," MANAGER-- requires a valid username/password ",servername,1)
   end /* do */
   when checkit=2  then do               /* use privileges provided by sre */
      return ' '
   end /* do */

   otherwise do
      if wordpos('SUPERUSER',privset)=0 then do

      'header add WWW-Authenticate: Basic Realm=<Manager>'  /* challenge */
        call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 3.0//EN">'
        call lineout tempfile, "<html><head><title>Not authorized </title>"
        call lineout tempfile, '</head><body bgcolor="#'||usecolor||'">'

        call lineout tempfile,' <strong> You do not have sre_manager rights.</strong> <pre>'
        if who2=saddr2 then
                call lineout tempfile,'<br> <Em> You may want to edit your configuration file. </em> '
        call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'

        call lineout tempfile,' </body> </html> '
        call lineout tempfile
         signal off error ; signal off syntax

        return sref_response('unauth'," You do not have configuration rights ",servername,1)
      end
   end
end
return ''               /* if here, it's okay */



/* convert to xxx,yyy,zzz */
insertcommas:procedure
parse arg asize
 il=length(asize)
 if il>3 then do
         oop=""
         do mm=il to 3 by -3
               tt=substr(asize,mm-2,3)
               if mm=il then
                  oop=tt
               else
                oop=tt||','||oop
         end /* do */
         if mm<>0 then oop=substr(asize,1,mm)||','||oop
         asize=oop
 end
 return asize

/*************************************************/
/* Check for the existence of a directory. Correctly identifies
   empty directories.
Usage:
   flag=dir_exists(a_directory)
where
   flag=1 if a_directory exists (it might be an empty directory )
   flag=0 if it doesn't exist
*/
dir_exists:procedure 
parse arg adir
ip=pos('.',adir)
if ip>0 then do /* strip out filename */
   ip2=lastpos('\',adir)
   adir=left(adir,ip2)
end /* do */
adir=strip(adir)
adir=strip(adir,'t','\')
nowdir=directory()
nowdrive=filespec('d',nowdir'\')
nowpath=filespec('p',nowdir'\')
adr=filespec('d',adir)
if adr='' then do
   if abbrev(adir,'\')=0 then 
       adir=nowdrive||nowpath||adir
   else
       adir=nowdrive||adir
end /* do */

foo=sysfiletree(adir,goo,'D')
if  goo.0>0  then return 1
return 0


/**************************************/
/* read data sent back by an html FORM declared with:
   enctype="multipart/form-data" method="POST"

Calling syntax:
   nentries=read_multipart(stuff,content_type)
  where
     stuff == the body of a POST request (i.e.; the 4th argument sent to
              sre-http addons)
     nentries == the number of entries found. If error, nentries=0
    
  and also the expose variable FORM_DATA is constructed.

The structure of FORM_DATA is:
  FORM_DATA.0 = # of entries (in this multipart submission)
  FORM_DATA.!list.j = space delimited list of "variable names" in part
                       j (j=1.. FORM_DATA.0)
     For each word in FORM_DATA.!list.j, there is FORM_DATA. tail.
     In particular, FORM_DATA.!aword.j, where !aword is an ! prepended
     to a word form the FORM_DATA.!list.j list.
     For example, in almost all cases, one of these words will be "NAME".
       Thus, FORM_DATA.!NAME.j = the "name" of this variable
  FORM_DATA.j  - the actual value of this part.

  Basically, a typical entry  will contain:
    FORM_DATA.!NAME.j and FORM_DATA.j
  which can be interpreted as the "name" of the variable and it's "value".
  However, sometimes other variables will be mentioned in the
  FORM_DATA.!LIST. In particular, file uploads will often have a
  FORM_DATA.!FILENAME.j, which is often the local name of 
  the file the client is uploading.


  Notes:
    * if an error occurs, a 0 is returmed, and FORM_DATA.!ERROR
      will contain an error message
    * a content-disposition entry, if found, is NOT included in FORM_DATA

*/
read_multipart_data:procedure expose form_data. manname
parse arg abody,atype

drop form_data.
form_data.=''

crlf='0d0a'x

/* is there a content-type request header ? */
if atype="" then do
   form_data.!error=" No  content-type  request header"
   return 0
end

parse var atype thetype ";" boog 'boundary=' abound    /* get the type */

if translate(thetype)<>"MULTIPART/FORM-DATA" then do
  form_data.!error="No  multipart/form-data in Content-type "
  return 0
end

if translate(thetype)<>"MULTIPART/FORM-DATA" then do
  form_data.!error=" upload error: No boundary in multipart/form-data header "
  return 0
end

abound="--"||abound   /* since boundaries always start with -- */

abd2=abound||crlf
/* loop through message, pulling out blocks and storing in stem var bigstuff. */

/* Now parse the various parts.*/

parse var abody foo1 (abd2) abody    /* move beyond first boundary and it's crlf */
/* check for netscape 2.0 incorrect format */
if pos(abound,abody)=0 then do   /* no ending boundary, so add one */
   abody=abody||crlf||abound||" -- "
end

mm=0
do until abody=""
  parse var abody thestuff (abound) abody        /* get a  boundary defined block */
  if strip(left(thestuff,4))="--" then leave        /* -- signals no more */
  if abody="" then leave
  mm=mm+1
  form_data.!list.mm='' ; form_data.mm=''
  do forever            /* get block headers.  Stop when hit a blank line */
     parse var thestuff anarg (crlf) thestuff
     if anarg="" then do
           leave
     end
     else do                    /* extract the arguments on this line */
         do until anarg=""
              parse var anarg anarg1 ";" anarg
              boob1=pos(':',anarg1) ; boob2=pos('=',anarg1)
              if boob1=0 then nixon=boob2
              if boob2=0 then nixon=boob1
              if boob1>0 & boob2>0 then nixon=min(boob1,boob2)
              t1=translate(strip(strip(substr(anarg1,1,nixon-1)),,'"'))
              t2=strip(strip(substr(anarg1,nixon+1)),,'"')
              if t1="CONTENT-DISPOSITION" then iterate /* don't bother retaining this */
              form_data.!list.mm=form_data.!list.mm' 't1
              nm1='!'||t1
              form_data.nm1.mm=t2
          end     /* exract arguments */
     end        /* extract args on this line */
  end                    /* get a line */
  if thestuff<>"" then do
    form_data.mm=left(thestuff,length(thestuff)-2)  /* strip off ending crlf */
    parse var abody foo (crlf) abody   /* jump past extra crlf */
  end
  else do
     form_data.body.mm=""
  end
end

return mm


/*********************************/
/* expand a .zip file (adapted from the GETAFILE add-on
zipfile: The zipfile to examine
backto: url for the backto arrow
lookin: real directory of the zipfile 
 */

show_zipfile:
thezip=strip(translate(varlist.!zipfile,'\','/'))
parse var thezip '$' atype '\' fname
atype=translate(atype)
aa='$'atype
addme=rootlist.aa
if addme='' then do
   call lineout tempfile,'<html><head><title>SRE-FileManager: view zipfile</title></head>'
   call lineout tempfile,' <b>Invalid root directory:</b> 'atype
   call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

zipfile=addme'\'fname

if is_allowed_dir(zipfile)=0 then return 1

rc=uzfiletree(zipfile,getem,,,'Z')
call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
call lineout tempfile, "<HTML>"
call lineout tempfile, "<HEAD>"
call lineout tempfile, "<TITLE>Contents of "zipfile"</TITLE>"
call lineout tempfile, "</HEAD> <body> <h2> Contents of " zipfile '</h2>'


call lineout tempfile, '<pre> <b>'left("Name",19)||left("Last Modified",17)||right("Size",8)'</b>'
call lineout tempfile,'<hr>    <b>Key:</b> <u><Em>+</em></u> = extract | <u><Em>++</em></u> : extract &amp; overwrite<br>'

call lineout tempfile,'<a href="javascript:self.back()">back</a><br>'
do mm=1 to getem.0
    aline=getem.mm
    Fname=word(aline,8)
    Ftime=word(aline,6)
    Fdate=word(aline,5)
    fdate=dateconv(translate(fdate,'/','-'),'U','N')
    Fsize=word(aline,1)
    feeb=' '
    feeb1=manname'?INZIP='fname'%26THEZIP='thezip
    feeb2=manname'?INZIP='fname'%26THEZIP='thezip'%26UNZIP=1'

    feeb3=feeb||'<A href="'||feeb1||'" target="VIEWER">'||fname||'</a> '
    feeb3=feeb3||' <em>(<A href="'||feeb2||'">+</a>)</em>'
    feeb3=feeb3||' <em>(<A href="'||feeb2||'%26over=1">++</a>)</em>'

    call lineout tempfile, feeb3''copies(' ',max(0,20-length(Fname)))''right(Fdate,10)''right(Ftime,6)' 'right(Fsize,10)
end

call lineout tempfile,' '

feeb1=manname'?UNZIP='thezip
call lineout tempfile,'</pre><form action="'manname'" method="GET">'
call lineout tempfile,'<input type="HIDDEN" name="UNZIP" value="'THEZIP'">'
call lineout tempfile,'<select size=5 name="OPT">'
call lineout tempfile,'<option value="-o">Overwrite everything'
call lineout tempfile,'<option value="-n">Never Overwrite '
call lineout tempfile,'<option value="-f">Freshen pre-existing'
call lineout tempfile,'<option value="-u">Update (freshen, and create)'
call lineout tempfile,'</select>'
call lineout tempfile,'<input type="checkbox" name="OPT2" value="-j">Do NOT make directories<br>'
call lineout tempfile,'<input type="submit" value="UnZip '||filespec('n',thezip)||'">'
call lineout tempfile,'</form>'

call lineout tempfile, "</BODY>"
call lineout tempfile, "</HTML>"
call lineout tempfile


return 


/*********************************/
/* unzip the file */

unzip_zipfile:
thezip=strip(translate(varlist.!unzip,'\','/'))
parse var thezip '$' atype '\' fname
atype=translate(atype)
aa='$'atype
addme=rootlist.aa
if addme='' then do
   call lineout tempfile,'<html><head><title>SRE-FileManager: unzip</title></head>'
   call lineout tempfile,' <b>Invalid root directory:</b> 'atype
   call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */

zipfile=addme'\'fname
call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
call lineout tempfile, "<HTML>"
call lineout tempfile, "<HEAD>"
call lineout tempfile, "<TITLE>UnZipping</TITLE>"
call lineout tempfile, '</HEAD> <body> <h2> Files unzipped from :'zipfile '</h2>'

if is_allowed_dir(zipfile)=0 then return 1

aopt=varlist.!opt
zipfile=translate(zipfile,'\','/')
if aopt='' then aopt=' -n'
tod=strip(filespec('d',zipfile)||filespec('p',zipfile),,'\')
tod='-d 'tod
fro=uzunzip(aopt||' '||tod||' '||varlist.!opt2||' '||zipfile,'outit.')
call lineout tempfile,'<pre> '
do mm=1 to outit.0
   call lineout tempfile,outit.mm
end /* do */
call lineout tempfile,'</pre>'

call lineout tempfile,'<a href="javascript:self.back()">back</a><br>'

call lineout tempfile, "</pre></BODY>"
call lineout tempfile, "</HTML>"
call lineout tempfile


return 0



/**************/
/* Extract and send a zip file.
zipfile: The "local" file to be unzipped
zipdir:  url-directory of the zipfile
lookin: the actual directory of the zipfile
*/
send_zipfile:

thezip=strip(translate(varlist.!thezip,'\','/'))
parse var thezip '$' atype '\' fname
atype=translate(atype)
aa='$'atype
addme=rootlist.aa
if addme='' then do
   call lineout tempfile,'<html><head><title>SRE-FileManager: file upload</title></head>'
   call lineout tempfile,' <b>Invalid root directory:</b> 'atype
   call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
   call lineout tempfile,'</body></html>'
   call lineout tempfile
   return 1
end /* do */
zipfile=addme'\'fname

if is_allowed_dir(zipfile)=0 then return 1

/* make sure it exists */
if stream(zipfile,'c','query exists')=' ' then  do
  call lineout tempfile, "<body><h2>Sorry...</h2>"
  call lineout tempfile,' Could not find .ZIP file: ' zipfile
  call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
  call lineout tempfile, "</body></html>"
  call lineout tempfile  /* close */
   oof=dosdir(tempfile,'s')
  return 'FILE ERASE TYPE text/html NAME ' tempfile
end
getfile=varlist.!inzip
rc=uzunziptostem(zipfile,'sook.',getfile)

if sook.1=0 then do
  call lineout tempfile, "<body><h2>Sorry...</h2>"
  call lineout tempfile,' Could not find Zipped file: ' getfile
  call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
  call lineout tempfile, "</body></html>"
  call lineout tempfile 
  return 1
end

arf=strip(sook.1)
thesize=length(sook.arf)
if thesize=0 then do  /* hack to get around unzip.dll ?bug? */
     sook.arf=uzunziptovar(zipfile,getfile)
     thesize=length(sook.arf)
end

if varlist.!unzip=1 then do                   /* sae the zipped file to disk */
    ggo=translate(getfile,' ','/\')
    getfile2=strip(word(ggo,words(ggo)))
    saveto=strip(filespec('d',zipfile)||filespec('p',zipfile),'t','\')'\'getfile2
    if varlist.!over<>1 then do
        ook=stream(saveto,'c','query exists')
        if ook<>'' then do
           call lineout tempfile,'<html><head><title>SRE-FileManager: zip extract</title></head>'
           call lineout tempfile,' <b>Ovewrite not permitted, can not extract:</b> 'saveto
           call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
           call lineout tempfile,'</body></html>'
           call lineout tempfile
           return 1
        end /* do */
    end /* do */
    foo=stream(saveto,'c','open write')
    if abbrev(translate(foo),'READY')<>1 then do
           call lineout tempfile,'<html><head><title>SRE-FileManager: zip extract</title></head>'
           call lineout tempfile,' <b>Can not write zip file to:</b> 'saveto
           call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
           call lineout tempfile,'</body></html>'
           call lineout tempfile
           return 1
    end
    fii=charout(saveto,sook.arf,1)
    if fii<>0 then do
           call lineout tempfile,'<html><head><title>SRE-FileManager: zip extract</title></head>'
           call lineout tempfile,' <b>Unable to fully write zip file to:</b> 'saveto
           call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
           call lineout tempfile,'</body></html>'
           call lineout tempfile
           return 1
     end 
     foo=stream(saveto,'c','close')
     call lineout tempfile,'<html><head><title>SRE-FileManager: zip extract</title></head>'
     call lineout tempfile,'<b>'length(sook.arf)' bytes  extracted to:</b> 'saveto
     call lineout tempfile,'<br><a href="javascript:self.back()">back</a><br>'
     call lineout tempfile,'</body></html>'
     call lineout tempfile
     return 1
end
else do              /* dsplay */
       atype=sref_mediatype(getfile)
       oo=value('sref_prefix',,'os2environment')
       if oo='' then do
         'VAR TYPE ' atype ' as ' getfile ' name sook.arf '
       end
       else do
         foog=sook.arf      /* sometime problems passing stem vars */
         foo=sref_gos('VAR TYPE ' atype ' as ' getfile ' name foog',foog)
       end
      return '200 '||length(sook.arf)
  end
end

/* here on error */
badguys:
string 'Error at ' sigl  ' ' rc
exit



/* here on no dynamic */
nodyn:
'string SRE-http manager: problem with dynamic password facility (it probably was not installed) '
exit




/* ================================== */
/* do a multipart/mixed or application/sre-mixed download */
do_downloads:

/* 3=multipart/mixed binary 2= multipart/mixed mime, 1=application/sre-mixed */
dtype=varlist.!download_type  

ifil=0
alist=varlist.!filelist
do forever
  if alist='' then leave
  parse var alist thefile alist
  a1=strip(a1)
  parse var thefile '$' atype '\' fname
  atype=translate(atype)
  aa='$'atype
  addme=rootlist.aa
  if addme<>'' then do
     ifil=ifil+1
     fils.ifil=addme'\'fname
     fils.0=ifil
  end
end

stuff='X-FileCount: 'fils.0||crlf


ii=lastpos('\',translate(fils.1,'\','/'))
if ii=0  then do
    ii=pos(':',fils.1)
end 
fname=substr(fils.1,ii+1)
atype='application/octet-stream'
if dtype<>3 then atype=sref_mediatype(fname)
if dtype=1 then do
  stuff=sref_multi_send2(fils.1,atype,'S',1,'Content-disposition: attachment; filename="'fname'"')
end
else do
  stuff=sref_multi_send(fils.1,atype,'S',1,,,'Content-disposition: attachment; filename="'fname'"')
end /* do */
if dtype=2 & datatype(varlist.!DELAY)="NUM" then call delay(varlist.!DELAY)


do mm=2 to fils.0-1
  ii=lastpos('\',translate(fils.mm,'\','/'))
  if ii=0  then do
     ii=pos(':',fils.mm)
  end /* do */
  fname=substr(fils.mm,ii+1)
  if dtype<>3 then atype=sref_mediatype(fname)
  if dtype=1 then do
     stuff=sref_multi_send2(fils.mm,atype,'M',1,'Content-disposition: attachment; filename="'fname'"')
  end
  else do
     stuff=sref_multi_send(fils.mm,atype,'M',1,,,'Content-disposition: attachment; filename="'fname'"')
  end /* do */
  if dtype=2 & datatype(varlist.!DELAY)="NUM" then call delay(varlist.!DELAY)

end /* do */


mm=fils.0
ii=lastpos('\',translate(fils.mm,'\','/'))
if ii=0  then do
     ii=pos(':',fils.mm)
end /* do */
fname=substr(fils.mm,ii+1)

if dtype<>3 then atype=sref_mediatype(fname)
if dtype=1 then do
  stuff=sref_multi_send2(fils.mm,atype,'E',1,'Content-disposition: attachment; filename="'fname'"')
end
else do
  stuff=sref_multi_send(fils.mm,atype,'E',1,,,'Content-disposition: attachment; filename="'fname'"')
end /* do */
if dtype=2 & datatype(varlist.!DELAY)="NUM" then call delay(varlist.!DELAY)


if dtype=1 then do
  resp=sref_gos('var type application/x-sre-mixed name foo ',stuff)
end

return 0



/* send a multipart document as one big document  -- cause netscape's got a bug ! */

sref_multi_send2:procedure expose stuff manname
parse arg thevalue,thetype,astatus,isfile,optheaders

parse var astatus status doclose

eclose=''

crlf='0d0a'x                /* useful */
bound='asfa235sdfF#234asg#%^6q32472341as'
bound2='asfA235sdfF#234asg#%^6q32472341p'

/* if isfile, then thevalue is a fully qualified file name. Read it */
if isfile=1 then do
      afil=stream(thevalue,'c','query exists')
      if afil="" then do
            thevalue=""
      end  /* Do */
      else do
         thevalue=charin(afil,1,chars(afil))
      end
end  /* Do */

/* if by odd chance a bound string is found in thevalue, it will
  be replaced by bound2 */
foo=pos(bound,thevalue)
if foo>0 then do
      thevalue=sref_replacestrg(thevalue,bound,bound2,'ALL') 
      call pmprintf('Multi_Send. Warning: boundary value found in message (and modified) ')
end

/* mime types */
if thetype=0 | thetype="" then thetype="text/html"

/* status */
status=strip(upper(status)); 

  mimestart='--'bound''crlf   /* starts a MIME multipart section */
  mimeend  ='--'bound'--'crlf /* ends a MIME multipart section */
  out=""

/* for now, assume multipart sends are not to be cached */
  if status='S'  then do
    stuff=stuff||'Content-Type: multipart/mixed;boundary='bound||crlf
    stuff=stuff||crlf||mimestart         
  end

  if wordpos(status,'S M E ')>0 then do   
     stuff=stuff||'Content-type: '||thetype||crlf
    if optheaders<>'' then stuff=stuff||optheaders||crlf
     stuff=stuff||crlf
  end

  out=out||thevalue||crlf

  if status="E"  then do
    out=out||mimeend           /* Mime 'last data' indicator */
  end
  if wordpos(status,'M S')>0 then do
       out=out||mimestart         /* Mime 'start data' indicator */
 end
 stuff=stuff||out

  return stuff


/***********************/
/* check dynamic password. A blank return means "okay" 
  Otherwise, return '200 '||length(response)

calling syntax:
  res=sref_dynamic_pwd(appname,newlocation,privset2,duration,respfile,clientaddr)
where
  appname: recommended. the "application" calling this. Used to identify 
           a "shared secret". If not specified, a value DYNPWD is used.
           Note that APPNAME is also used in responses.
  newlocation: recommended. The uri to invoke when client hits the submit
               button. Default value is "/"
  privset2 : REQUIRED. the space delimited list of "secret" privileges. 
             It should contain an entry of the form APPNAME:shared_secret
  duration: optional. duration of a timestamp, in days (and fractions of day)
             (default value is 1 day)
  respfile: optional.the "response file" containing the md5 procedure. If not
            specified, DYNPWD.HTM, in the (possibly host-specific) data-directory
  clientaddr: client's numeric ip address. Optional, if missing, will get if
            with a GOSERVE extract 
and
  resp='' 
         if a valid password is found
or 
  resp='200 '||length(response) 
        if no valid password is found, 
        sref_dynamic_pwd asks the client to supply one -- this is the
        response code for this "response"
*/
sref_dynamic_pwd:procedure

parse  arg appname,newlocation,privset2,duration,respfile,who

tappname=translate(strip(appname))
if tappname='' then tapname='DYNPWD'

if newlocation='' then newlocation='/'
if datatype(duration)<>'NUM' then duration=1

/*response file, relative to datadir? */
if respfile='' then respfile='DYNPWD.RSP'
respfile=strip(translate(strip(respfile),'\','/'),'l','\')
if pos(':',respfile)=0  then do
   dd=sref_value('DATADIR',,'req')
   respfile=strip(translate(dd,'\','/'),'t','\')||'\'||respfile
end

nowtime=date('b')+(time('m')/(24*60))
if who='' then who=extract('clientaddr')

/* 1) get the  APPNAME password for this user (it's stored in
      a "secret" privilege that begins with  APPNAME: */
sspwd=''
do mm=1 to words(privset2)
   aw=strip(translate(word(privset2,mm)))
   if abbrev(aw,tappname||':')=1 then do
        sspwd=strip(translate(substr(aw,length(tappname)+2))); leave
   end /* do */
end /* do */

/* 2) If no APPNAME password (the shared secret), ask client to logon again */
if sspwd='' then do  /* dynamic pwd REQUIRES a appname:xxx secret privilege */
  'header add WWW-Authenticate: Basic Realm=<'appname'>'  /* challenge */
   servername=servername()
   return sref_response('unauth',' A username/password does not have 'appname' permissions ',servername,1)
end

/* 3) a appname: password exists for this client, now get some info from cookies */

/* 3a) get timestamp */
dynpwd=sref_get_cookie(appname)
if dynpwd='' then do            /* no dynamic password */
  foo=sref_dynamic_pwd2(appname,newlocation,who,nowtime,' Please enter a session key',respfile)
  return foo
end
parse var dynpwd tstamp '_' dynpwd

/* 3b) is timestamp valid */
if datatype(tstamp)<>'NUM' then do
  foo=sref_dynamic_pwd2(appname,newlocation,who,nowtime,'Your session key was invalid ',respfile)
  return foo
end

/* 3c)  timestamp expired? */
if (nowtime-duration)>tstamp then do
  foo=sref_dynamic_pwd2(apppname,newlocation,who,nowtime,'Your prior session key has expired',respfile)  
  return foo
end

/* 3d) dynamic password not empty */
if dynpwd='' then do
  foo=sref_dynamic_pwd2(appname,newlocation,who,nowtime,' Your session key was invalid',respfile)
  return foo
end

/* 4) generate what the hash should be: concatenate
     ip-address, timestamp, shared-secret; and use it's md5 */

thekey=strip(who)||'_'||strip(tstamp)||'_'||sspwd
thekey2=strip(translate(sref_md5(thekey,0)))

/* 5) does required "hash" match the "dynamic password" supplied by the client */
if translate(strip(dynpwd))<>thekey2 then do
  foo=sref_dynamic_pwd2(appname,newlocation,who,nowtime,'Your password did not work -- please reenter ',respfile)  
  return foo
end

return ''                       /* legit! */

/***********************/
/* Ask the client to provide a  "dynamic" password */
/* This requires the md5 javascript script, as contained 
   in  respfile. If respfile= '', use datadir\dynpwd.htm' */

sref_dynamic_pwd2:procedure
parse arg  appname,newlocation,who,tstamp,message,respfile

crlf='0a0a'x


/* does it exist? */
iin=stream(respfile,'c','query size')
if iin=0 | iin='' then do
       aa="<html><head><title>ERROR:'amessage'</title>"crlf
       aa=aa'</head><body><h3>ERROR:'amessage'</h3>'crlf
       aa=aa' Missing response file ('respfile||'</body></html>'
       'var type text/html name aa'
       return '200 '||length(aa)         
end /* do */

foo=stream(respfile,'c','open read')
if abbrev(translate(foo),'READY')=0 then do
       aa="<html><head><title>ERROR:'amessage'</title>"crlf
       aa=aa'</head><body><h3>ERROR:'amessage'</h3>'crlf
       aa=aa' Unable to open response file ('respfile||'</body></html>'
       'var type text/html name aa'
       return '200 '||length(aa)         
end

stuff=charin(respfile,1,iin)

foo=stream(respfile,'c','close')


aa="<html><head><title>'amessage'</title>"crlf
aa=aa'</head><body><h3>'amessage'</h3>'crlf

/* change these lines in respfie 
    var appname="##APPNAME"          // the application that needs a dynamic password
    var theip="##THEIP"              // the client's ip address
    var tstamp="##TSTAMP"            // a time stamp 
    var newlocation="##NEWLOCATION"  // go here on submission
*/
stuff=sref_replacestrg(stuff,'##MESSAGE',message)
stuff=sref_replacestrg(stuff,'##APPNAME',appname)
stuff=sref_replacestrg(stuff,'##THEIP',who)
stuff=sref_replacestrg(stuff,'##TSTAMP',tstamp)
stuff=sref_replacestrg(stuff,'##NEWLOCATION',newlocation)

'var type text/html name stuff'

return '200 '||length(stuff)



